/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/

#include "TeThemeApplication.h"
#include "TeUtils.h"
#include "TeDatabase.h"
#include "TeApplicationUtils.h"

#include <TeQtCanvas.h>


TeThemeApplication::TeThemeApplication(TeThemeApplication& other) :
     TeTheme(other.name(), other.layer(), other.parent(), other.view(), other.id())
{
	outOfCollectionVisibility_ = other.outOfCollectionVisibility_;
	withoutDataConnectionVisibility_ = other.withoutDataConnectionVisibility_;
	refineAttributeWhere_ = other.refineAttributeWhere_;
	refineSpatialWhere_ = other.refineSpatialWhere_; 
	refineTemporalWhere_ = other.refineTemporalWhere_;
	groupColor_ = other.groupColor_;
	groupColorDir_ = other.groupColorDir_;
	groupObjects_ = other.groupObjects_;
	pieDiameter_ = other.pieDiameter_;
	pieMinDiameter_ = other.pieMinDiameter_;
	pieMaxDiameter_ = other.pieMaxDiameter_;
	barMinHeight_ = other.barMinHeight_;
	barMaxHeight_ = other.barMaxHeight_;
	barWidth_ = other.barWidth_;
	chartMaxVal_ = other.chartMaxVal_;
	chartMinVal_ = other.chartMinVal_;
	chartType_ = other.chartType_;
	chartObjects_ = other.chartObjects_;
	chartFunction_ = other.chartFunction_;
	keepDimension_ = other.keepDimension_;
	showCanvasLegend_ = other.showCanvasLegend_;
	canvasLegWidth_ = other.canvasLegWidth_;
	canvasLegHeight_ = other.canvasLegHeight_; 
	editRep_ = other.editRep_;
	mix_color_ = other.mix_color_;
	count_obj_ = other.count_obj_; 
	textTable_ = other.textTable_;
	uidStatusMap_ = other.uidStatusMap_;
}

 TeThemeApplication& TeThemeApplication::operator =(const TeThemeApplication& other)
{
	if (this != &other)
	{
		TeTheme *th = (TeTheme*)this;
		TeTheme *oth = (TeTheme*)&other;
		*th = *oth;
		outOfCollectionVisibility_ = other.outOfCollectionVisibility_;
		withoutDataConnectionVisibility_ = other.withoutDataConnectionVisibility_;
		refineAttributeWhere_ = other.refineAttributeWhere_;
		refineSpatialWhere_ = other.refineSpatialWhere_; 
		refineTemporalWhere_ = other.refineTemporalWhere_;
		groupColor_ = other.groupColor_;
		groupColorDir_ = other.groupColorDir_;
		groupObjects_ = other.groupObjects_;
		pieDiameter_ = other.pieDiameter_;
		pieMinDiameter_ = other.pieMinDiameter_;
		pieMaxDiameter_ = other.pieMaxDiameter_;
		barMinHeight_ = other.barMinHeight_;
		barMaxHeight_ = other.barMaxHeight_;
		barWidth_ = other.barWidth_;
		chartMaxVal_ = other.chartMaxVal_;
		chartMinVal_ = other.chartMinVal_;
		chartType_ = other.chartType_;
		chartObjects_ = other.chartObjects_;
		chartFunction_ = other.chartFunction_;
		keepDimension_ = other.keepDimension_;
		showCanvasLegend_ = other.showCanvasLegend_;
		canvasLegWidth_ = other.canvasLegWidth_;
		canvasLegHeight_ = other.canvasLegHeight_; 
		editRep_ = other.editRep_;
		mix_color_ = other.mix_color_;
		count_obj_ = other.count_obj_; 
		textTable_ = other.textTable_;
		uidStatusMap_ = other.uidStatusMap_;
	}

	return *this;
}


unsigned int TeThemeApplication::getTableIndexFromField(string fieldName)
{
	unsigned int i, n, index;
	index = 0;
	n = attTableVector_[0].attributeList().size();
	for (i = 0; i < sqlAttList_.size(); ++i)
	{
		if (i > n - 1)
		{
			++index;
			n += attTableVector_[index].attributeList().size();
		}

		if (sqlAttList_[i].rep_.name_ == fieldName)
			break;
	}
	return index;
}



bool TeThemeApplication::locateText (TeCoord2D &pt, TeText &text, const double& tol)
{
	TeDatabasePortal* portal = layer()->database()->getPortal();
	string table = textTable();

	TeBox box (pt.x()-tol,pt.y()-tol,pt.x()+tol,pt.y()+tol);
	string q = "SELECT " + table + ".* FROM " + table + ", " + collectionTable();
	char buf [1024];
	sprintf (buf," WHERE c_object_id = object_id AND (x < %f AND x > %f AND y < %f AND y > %f)",
		box.x2(),box.x1(),box.y2(),box.y1());
	q += buf;
	if (!portal->query(q) || !portal->fetchRow())
	{
		delete portal;
		return false;
	}
	// Look for all texts
	bool flag = true;
	TeTextSet ts;
	do
	{
		TeText p;
		flag = portal->fetchGeometry(p);
		ts.add ( p );
	} while (flag);

	delete portal;

	int k;
	if (TeNearest (pt, ts, k, tol))
	{
		text = ts[k];
		return true;
	}
	return false;
}

TeVisual TeThemeApplication::getTextVisual(const TeText &text)
{
	int	dot_height, rgb;
	string family;
	string table = textTable() + "_txvisual";
	bool fix_size = false, bold = false, italic = false;
	TeDatabase* db = layer()->database();

	string sel = "SELECT dot_height, fix_size, color, family, bold, italic FROM ";
	sel += table + " WHERE geom_id = " + Te2String(text.geomId());

	TeDatabasePortal* portal = db->getPortal();
	if(portal->query(sel))
	{
		if(portal->fetchRow())
		{
			dot_height = atoi(portal->getData(0));
			if(atoi(portal->getData(1)))
				fix_size = true;
			rgb = atoi(portal->getData(2));
			family = portal->getData(3);
			if(atoi(portal->getData(4)))
				bold = true;
			if(atoi(portal->getData(5)))
				italic = true;
		}
	}
	delete portal;
	TeColor	cor((rgb >> 16) & 0xff, (rgb >> 8) & 0xff, rgb & 0xff);
	TeVisual visual;
	visual.size(dot_height);
	visual.fixedSize(fix_size);
	visual.color(cor);
	visual.family(family);
	visual.bold(bold);
	visual.italic(italic);

	return visual;
}


string TeThemeApplication::getLegendTitleAlias()
{
	string text;

	TeLegendEntryVector& legendVector = legend();
	if (legendVector.size() > 0)
	{
		TeDatabase* database = layer()->database();
		TeAttributeRep rep; 
		string nattr;
		if(grouping().groupMode_ != TeNoGrouping)
		{
			rep = grouping().groupAttribute_; 
			nattr = grouping().groupNormAttribute_;
		}
		text = rep.name_;
		map<int, map<string, string> >::iterator it = database->mapThemeAlias().find(id());
		if(it != database->mapThemeAlias().end())
		{
			map<string, string>& m = it->second;
			map<string, string>::iterator tit = m.find(text);
			if(tit != m.end())
				text = tit->second;
			if(!(nattr.empty() || nattr == "NONE"))
			{
				tit = m.find(nattr);
        		if(tit != m.end()) {
        		  if (!tit->second.empty()) {
					text += "/" + tit->second;
          		  }
        		}
        		else  //Deixando ficar sem barra 
					text += "/" + nattr;
			}
		}
	}
	return text;
}

string TeThemeApplication::	getPieBarAlias(string atr)
{
	string text;

	if (chartAttributes_.size() > 0)
	{
		TeDatabase* database = layer()->database();
		map<int, map<string, string> >::iterator it = database->mapThemeAlias().find(id());
		if(it != database->mapThemeAlias().end())
		{
			map<string, string>& m = it->second;
			map<string, string>::iterator tit = m.find(atr);
			if(tit != m.end())
				text = tit->second;
		}
	}
	return text;
}

bool TeThemeApplication::setAlias(string attr, string attAlias)
{
	TeDatabase* db = layer()->database();
	map<int, map<string, string> >& mapThemeAlias = db->mapThemeAlias();
	map<string, string>& mapAA = mapThemeAlias[id()];
	map<string, string>::iterator it = mapAA.begin();

	while(it != mapAA.end())
	{
		if(it->first != attr && it->second == attAlias)
			return false;
		it++;
	}

	int f = attr.find("/");
	int af = attAlias.find("/");
	if(f >= 0 && af >= 0)
	{
		string nattr = attr;
		attr = attr.substr(0, f);
		nattr = nattr.substr(f+1);

		string alias = attAlias;
		string nalias = alias;
		alias = alias.substr(0, af);
		nalias = nalias.substr(af+1);

		mapAA[attr] = alias;
		mapAA[nattr] = nalias;
		updateThemeApplication (db, this);
		return true;
	}
	else
	{
    	//Alterando para deixar ficar sem /
    	if(f >= 0) {
      		string nattr = attr;
      		nattr = nattr.substr(f+1);
      		mapAA[nattr] = "";
			attr = attr.substr(0, f);
    	}
		mapAA[attr] = attAlias;
		updateThemeApplication (db, this);
		return true;
	}
}

void TeThemeApplication::removeAlias(string attribute)
{
	string attr = attribute;
	string nattr = attr;
	TeDatabase* db = layer()->database();
	map<int, map<string, string> >& mapThemeAlias = db->mapThemeAlias();
	map<string, string>& mapAA = mapThemeAlias[id()];

	int f = attr.find("/");
	if(f >= 0)
	{
		attr = attr.substr(0, f);
		nattr = nattr.substr(f+1);
	}
	map<string, string>::iterator it = mapAA.find(attr);
	if(it != mapAA.end())
	{
		mapAA.erase(it);
		if(mapAA.size() == 0)
		{
			map<int, map<string, string> >::iterator tit = mapThemeAlias.find(id());
			if(tit != mapThemeAlias.end())
				mapThemeAlias.erase(tit);
		}
	}
	if(nattr != attr)
	{
		map<string, string>::iterator it = mapAA.find(nattr);
		if(it != mapAA.end())
		{
			mapAA.erase(it);
			if(mapAA.size() == 0)
			{
				map<int, map<string, string> >::iterator tit = mapThemeAlias.find(id());
				if(tit != mapThemeAlias.end())
					mapThemeAlias.erase(tit);
			}
		}
	}

	updateThemeApplication (db, this);
}

void TeThemeApplication::removeAllAlias()
{
	TeDatabase* db = layer()->database();
	map<int, map<string, string> >& mapThemeAlias = db->mapThemeAlias();
	map<int, map<string, string> >::iterator it = mapThemeAlias.find(id());
	if(it != mapThemeAlias.end())
		mapThemeAlias.erase(it);
	updateThemeApplication (db, this);
}

void TeThemeApplication::concatTableName(string& attr)
{
	string tableName;

	if(attr.find(".") == attr.npos)
	{
		TeAttrTableVector::iterator it = attTableVector_.begin();

		while(it != attTableVector_.end())
		{
			TeTable	t = *it;
			TeAttributeList AL = t.attributeList();
			TeAttributeList::iterator iit = AL.begin();

			while(iit != AL.end())
			{
				if((*iit).rep_.name_ == attr)
				{
					tableName = (*it).name();
					break;
				}
				iit++;
			}

			if(tableName.empty() == false)
			{
				tableName += ".";
				attr.insert(0, tableName);
				break;
			}
			it++;
		}
	}
}



int TeThemeApplication::numObjects() {

  int numRows;

  string s = "SELECT COUNT(*) " + sqlFrom();	
  TeDatabasePortal* portal = layer_->database()->getPortal();
  if(portal->query(s))
  {
    if(portal->fetchRow())
    {
      numRows = atoi(portal->getData(0));
    }
  }
  
  delete portal;
  return numRows;
  
}

TeBox TeThemeApplication::boundingBox() {
    return layer_->database()->getThemeBox(this);
}

/*
TeBox TeThemeApplication::getPieBarBox(TeQtCanvas* canvas)
{
	TeBox	boxOut;

	if(chartAttributes_.size() == 0)
		return boxOut;

	string objectId = chartSelected_;

	string	TS = collectionTable();
	string	sel = "SELECT * FROM " + TS + " WHERE ";

//	if(chartObjects() == TeSelectedByPointing)
//		sel += TS + ".sel_by_pointing = 1 AND ";
//	else if(chartObjects() == TeNotSelectedByPointing)
//		sel += TS + ".sel_by_pointing = 0 AND ";
//	else if(chartObjects() == TeSelectedByQuery)
//		sel += TS + ".sel_by_query = 1 AND ";
//	else if(chartObjects() == TeNotSelectedByQuery)
//		sel += TS + ".sel_by_query = 0 AND ";
//	else if(chartObjects() == TeGrouped)
//		sel += TS + ".c_legend_id <> 0 AND ";
//	else if(chartObjects() == TeNotGrouped)
//		sel += TS + ".c_legend_id = 0 AND ";
	
	sel += TS + ".c_object_id = '" + objectId + "'";

	TeDatabasePortal *portal = layer()->database()->getPortal();
	if (!portal)
		return false;

	if (!portal->query(sel) || !portal->fetchRow())
	{
		delete portal;
		return boxOut;
	}

	canvas->setDataProjection(layer()->projection());

	double	width = barWidth_;
	double	maxh = barMaxHeight_;
	double	minh = barMinHeight_;
	double	maxd = pieMaxDiameter_;
	double	mind = pieMinDiameter_;
	double	diameter = pieDiameter_;

	if(keepDimension() != 0)
	{
		width = canvas->mapVtoCW((int)barWidth_);
		maxh = canvas->mapVtoCW((int)barMaxHeight_);
		minh = canvas->mapVtoCW((int)barMinHeight_);
		maxd = canvas->mapVtoCW((int)pieMaxDiameter_);
		mind = canvas->mapVtoCW((int)pieMinDiameter_);
		diameter = canvas->mapVtoCW((int)pieDiameter_);
	}

	unsigned int i;

	if(chartType_ == TePieChart)
	{
		double delta = diameter / 2.;
		if(!(pieAttr_ == "NONE"))
			delta = maxd / 2.;

		string attrs;
		for(i=0; i<chartAttributes_.size(); i++)
			attrs += chartAttributes_[i] + ",";
		attrs += "label_x,label_y";
		if(!(pieAttr_ == "NONE"))
			attrs += "," + pieAttr_;

		string q = "SELECT " + attrs + sqlFrom() + " WHERE " + TS + ".c_object_id = '" + objectId + "'";
		portal->freeResult();
		if(portal->query(q))
		{
			if(portal->fetchRow())
			{
				double tot = 0.;
				for(i=0; i<chartAttributes_.size(); i++)
				{
					string v = portal->getData(i);
					if(v.empty())
					{
						delete portal;
						return boxOut;
					}
					tot += atof(v.c_str());
				}
				if(tot == 0.)
				{
					delete portal;
					return boxOut;
				}

				if(!(pieAttr_ == "NONE"))
				{
					if(chartMaxVal_ - chartMinVal_ == 0)
						diameter = 0.;
					else
					{
						double	adim = portal->getDouble(chartAttributes_.size()+2);
						double a = (adim - chartMinVal_) / (chartMaxVal_ - chartMinVal_);
						diameter = a * (maxd - mind) + mind;
					}
				}
				double	x = chartPoint_.x();
				double	y = chartPoint_.y();
				boxOut.x1_ = x - diameter / 2.;
				boxOut.y1_ = y - diameter / 2.;
				boxOut.x2_ = x + diameter / 2.;
				boxOut.y2_ = y + diameter / 2.;
			}
		}
	}
	else
	{
		double	n = chartAttributes_.size();
		string attrs;
		for(i=0; i<chartAttributes_.size(); i++)
			attrs += chartAttributes_[i] + ",";
		attrs += "label_x,label_y";

		string q = "SELECT " + attrs + sqlFrom() + " WHERE " + TS + ".c_object_id = '" + objectId + "'";

		vector<double> dvec;
		portal->freeResult();
		if(portal->query(q))
		{
			if(portal->fetchRow())
			{
				double	x = chartPoint_.x();
				double	xmin = x - n * width / 2.;
				double	xmax = x + n * width / 2.;
				double	y = chartPoint_.y();

				dvec.clear();
				for(i=0; i<chartAttributes_.size(); i++)
				{
					string val = portal->getData(i);
					if(val.empty())
					{
						delete portal;
						return boxOut;
					}
					dvec.push_back(atof(val.c_str()));
				}

				double mheight = -10;
				for(i=0; i<dvec.size(); i++)
				{
					double height;
					double v = dvec[i];
					if(chartMaxVal_ - chartMinVal_ == 0)
						height = 0.;
					else
						height = (v - chartMinVal_) * (maxh - minh) / (chartMaxVal_ - chartMinVal_) + minh;
					mheight = MAX(height, mheight);
				}
				boxOut.x1_ = xmin;
				boxOut.y1_ = y;
				boxOut.x2_ = xmax;
				boxOut.y2_ = y + mheight;
			}
		}
	}
	delete portal;
	return boxOut;
}*/


void TeThemeApplication::setStatusForObjectToggled(string oid)
{
	int oidStatus = 0;
	int uidStatus;
	string uid;

	vector<string> uidVec = getUidVec(oid, this);
	for (unsigned int i = 0; i < uidVec.size(); ++i)
	{
		uid = uidVec[i];
		uidStatus = uidStatusMap_[uidVec[i]];
		if (uidStatus == TeDEFAULT)
			uidStatus = TePOINTED;
		else if (uidStatus == TePOINTED)
			uidStatus = TeDEFAULT;
		else if (uidStatus == TeQUERIED)
			uidStatus = TePOINTED_QUERIED;
		else if (uidStatus == TePOINTED_QUERIED)
			uidStatus = TeQUERIED;

		oidStatus = MAX(oidStatus, uidStatus);
		uidStatusMap_[uidVec[i]] = uidStatus;
	}
	
	// Save the status id of the object
	if (oidStatus == 0)
		oidStatusMap_.erase(oid);
	else
		oidStatusMap_[oid] = oidStatus;
}

void TeThemeApplication::setStatusForUidsToggled(set<string>& oidSet, vector<string>& uidVec)
{
	unsigned int i;
	int oidStatus;
	string oid, uid;
	set<string>::iterator it;

	// Set the new status of the uids
	for (i = 0; i < uidVec.size(); ++i)
	{
		uid = uidVec[i];
		if (uidStatusMap_[uid] == TeDEFAULT)
			uidStatusMap_[uid] = TePOINTED;
		else if (uidStatusMap_[uid] == TePOINTED)
			uidStatusMap_.erase(uid);
		else if (uidStatusMap_[uid] == TeQUERIED)
			uidStatusMap_[uid] = TePOINTED_QUERIED;
		else if (uidStatusMap_[uid] == TePOINTED_QUERIED)
			uidStatusMap_[uid] = TeQUERIED;
	}

	// Set the new status of the oids
	for (it = oidSet.begin(); it != oidSet.end(); ++it)
	{
		oid = *it;
		oidStatus = 0;
		vector<string> uidVec = getUidVec(oid, this);
		for (i = 0; i < uidVec.size(); ++i)
		{
			uid = uidVec[i];
			if (uidStatusMap_[uid] == TePOINTED_QUERIED)
			{
				oidStatus = TePOINTED_QUERIED;
				break;
			}
			oidStatus = MAX(oidStatus, uidStatusMap_[uid]);
			if (uidStatusMap_[uid] == TeDEFAULT)
				uidStatusMap_.erase(uid);
		}
		
		if (oidStatus == 0)
			oidStatusMap_.erase(oid);
		else
			oidStatusMap_[oid] = oidStatus;
	}
}

void TeThemeApplication::setStatusForNewQuery(set<string>& oidSet, vector<string>& uidVec)
{
	unsigned int i;
	string oid, uid;
	vector<string> prevOidQVec;		// previous oids queried
	vector<string> notInOidSetVec;	// oid is not in oidSet
	vector<string> defaultVec;		// vector with oids or uids to be set as default
	map<string, int>::iterator mapIt;
	set<string>::iterator setIt;

	// Get the previous oids queried
	for (mapIt = oidStatusMap_.begin(); mapIt != oidStatusMap_.end(); ++mapIt)
	{
		oid = mapIt->first;
		if (oidStatusMap_[oid] == TeQUERIED || oidStatusMap_[oid] == TePOINTED_QUERIED)
			prevOidQVec.push_back(mapIt->first);
	}

	// Check if the previous oids queried are in the oidSet; in positive case, remove
	// them from the oidSet; in negative case, put them in the vector notInOidSetVec
	for (i = 0; i < prevOidQVec.size(); ++i)
	{
		oid = prevOidQVec[i];
		if (oidSet.find(oid) != oidSet.end())			
			oidSet.erase(oid);				// object is in the input oidSet
		else
			notInOidSetVec.push_back(oid);	// object is not in the input oidSet
	}

	// For the objects in oidSet, set the queried status for them
	for (setIt = oidSet.begin(); setIt != oidSet.end(); ++setIt)
	{
		oid = *setIt;
		if (oidStatusMap_[oid] == TeDEFAULT)
			oidStatusMap_[oid] = TeQUERIED;
		else if (oidStatusMap_[oid] == TePOINTED)
			oidStatusMap_[oid] = TePOINTED_QUERIED;
	}

	// For the objects not in oidSet, insert it in oidSet, and remove their queried status
	for (i = 0; i < notInOidSetVec.size(); ++i)
	{
		oid = notInOidSetVec[i];
		if (oidStatusMap_[oid] == TeQUERIED)
			defaultVec.push_back(oid);
		else if (oidStatusMap_[oid] == TePOINTED_QUERIED)
			oidStatusMap_[oid] = TePOINTED;
		
		oidSet.insert(oid);
	}

	for (i = 0; i < defaultVec.size(); ++i)
		oidStatusMap_.erase(defaultVec[i]);

	// Remove the queried status for the uids that were previously queried
	defaultVec.clear();
	for (mapIt = uidStatusMap_.begin(); mapIt != uidStatusMap_.end(); ++mapIt)
	{
		uid = mapIt->first;
		if (uidStatusMap_[uid] == TeQUERIED)
			defaultVec.push_back(uid);
		else if (uidStatusMap_[uid] == TePOINTED_QUERIED)
			uidStatusMap_[uid] = TePOINTED;			
	}

	for (i = 0; i < defaultVec.size(); ++i)
		uidStatusMap_.erase(defaultVec[i]);

	// Set the queried status for the input uidVec
	for (i = 0; i < uidVec.size(); ++i)
	{
		uid = uidVec[i];
		if (uidStatusMap_[uid] == TeDEFAULT)
			uidStatusMap_[uid] = TeQUERIED;
		else if (uidStatusMap_[uid] == TePOINTED)
			uidStatusMap_[uid] = TePOINTED_QUERIED;
	}
}

void TeThemeApplication::setStatusForAddToQuery(set<string>& oidSet, vector<string>& uidVec)
{
	unsigned int i;
	string oid, uid;
	set<string>::iterator setIt;

	// Set the status for the oids in oidSet as "queried"
	for (setIt = oidSet.begin(); setIt != oidSet.end(); ++setIt)
	{
		oid = *setIt;
		if (oidStatusMap_[oid] == TeDEFAULT)
			oidStatusMap_[oid] = TeQUERIED;
		else if (oidStatusMap_[oid] == TePOINTED)
			oidStatusMap_[oid] = TePOINTED_QUERIED;
	}

	// Set the new status for the uidVec
	for (i = 0; i < uidVec.size(); ++i)
	{
		uid = uidVec[i];
		if (uidStatusMap_[uid] == TeDEFAULT)
			uidStatusMap_[uid] = TeQUERIED;
		else if (uidStatusMap_[uid] == TePOINTED)
			oidStatusMap_[uid] = TePOINTED_QUERIED;
	}
}


void TeThemeApplication::setStatusForFilterQuery(set<string>& oidSet, vector<string>& uidVec)
{
	unsigned int i;
	string oid, uid;
	vector<string> prevOidQVec;		// previous oids queried
	vector<string> notInOidSetVec;	// oid is not in oidSet
	vector<string> defaultVec;		// vector with oids or uids to be set as default
	map<string, int>::iterator mapIt;
	set<string>::iterator setIt;

	// Get the previous oids queried
	for (mapIt = oidStatusMap_.begin(); mapIt != oidStatusMap_.end(); ++mapIt)
	{
		oid = mapIt->first;
		if (oidStatusMap_[oid] == TeQUERIED || oidStatusMap_[oid] == TePOINTED_QUERIED)
			prevOidQVec.push_back(mapIt->first);
	}

	// Check if the previous oids queried are in the oidSet; in positive case, remove
	// them from the oidSet; in negative case, put them in the vector notInOidSetVec
	for (i = 0; i < prevOidQVec.size(); ++i)
	{
		oid = prevOidQVec[i];
		if (oidSet.find(oid) != oidSet.end())			
			oidSet.erase(oid);				// object is in the input oidSet
		else
			notInOidSetVec.push_back(oid);	// object is not in the input oidSet
	}

	// For the objects not in oidSet, insert it in oidSet, and remove their queried status
	for (i = 0; i < notInOidSetVec.size(); ++i)
	{
		oid = notInOidSetVec[i];
		if (oidStatusMap_[oid] == TeQUERIED)
			defaultVec.push_back(oid);
		else if (oidStatusMap_[oid] == TePOINTED_QUERIED)
			oidStatusMap_[oid] = TePOINTED;
		
		oidSet.insert(oid);
	}

	for (i = 0; i < defaultVec.size(); ++i)
		oidStatusMap_.erase(defaultVec[i]);

	// Remove the queried status for the uids that were previously queried
	defaultVec.clear();
	for (mapIt = uidStatusMap_.begin(); mapIt != uidStatusMap_.end(); ++mapIt)
	{
		uid = mapIt->first;
		if (uidStatusMap_[uid] == TeQUERIED)
			defaultVec.push_back(uid);
		else if (uidStatusMap_[uid] == TePOINTED_QUERIED)
			uidStatusMap_[uid] = TePOINTED;			
	}

	for (i = 0; i < defaultVec.size(); ++i)
		uidStatusMap_.erase(defaultVec[i]);

	// Set the queried status for the input uidVec
	for (i = 0; i < uidVec.size(); ++i)
	{
		uid = uidVec[i];
		if (uidStatusMap_[uid] == TeDEFAULT)
			uidStatusMap_[uid] = TeQUERIED;
		else if (uidStatusMap_[uid] == TePOINTED)
			uidStatusMap_[uid] = TePOINTED_QUERIED;
	}
}



