#include "TePDIAlgorithm.hpp"

#include "TePDIAgnostic.hpp"

TePDIAlgorithm::TePDIAlgorithm()
{
  pi_manager_enabled_ = true;
}


TePDIAlgorithm::~TePDIAlgorithm()
{
}


bool TePDIAlgorithm::Reset( const TePDIParameters& params )
{
  if( CheckParameters( params ) ) {
    ResetState( params );

    /* This line must come after the ResetState line, because some algorithms
       checks the difference between the current internal parameters and the
       new supplied parameters */
    params_.Clear();
    params_ = params;

    return true;
  } else {
    PDIAGN_LOG_AND_RETURN( "Invalid supplied parameters" );
  }
}


bool TePDIAlgorithm::Apply()
{
  bool return_value = false;
  
  if( CheckInternalParameters() ) {
    return_value = RunImplementation();
    StopProgInt();
  } else {
    PDIAGN_LOG_AND_RETURN( "Invalid supplied parameters" );
  }
  
  return return_value;
}


TePDIParameters TePDIAlgorithm::GetParameters() const
{
  return params_;
}


bool TePDIAlgorithm::CheckInternalParameters() const
{
  return CheckParameters( params_ );
}


const TePDIAlgorithm& TePDIAlgorithm::operator=( 
  const TePDIAlgorithm& external )
{
  PDIAGN_LOG_AND_THROW( "Algorithms cannot be copied" );  
  
  return external;
}

TePDIAlgorithm* TePDIAlgorithm::DefaultObject( const TePDIParameters& params )
{
  PDIAGN_LOG_AND_THROW( "Trying to create an invalid algorithm instance" );
  
  TePDIParameters dummy_params = params;

  return 0;
}


void TePDIAlgorithm::ToggleProgInt( bool enabled )
{
  pi_manager_enabled_ = enabled;
  
  if( ! enabled ) {
    pi_manager_.Toggle( false );
  }
}


void TePDIAlgorithm::StartProgInt( const std::string& message, 
  unsigned int steps )
{
  if( pi_manager_enabled_ ) {
    pi_manager_.Toggle( true );
    pi_manager_.Reset( message, steps );
  }
}


void TePDIAlgorithm::UpdateProgInt( unsigned int step )
{
  pi_manager_.Update( step );
}


void TePDIAlgorithm::IncProgInt()
{
  pi_manager_.Increment();
}


void TePDIAlgorithm::StopProgInt()
{
  pi_manager_.Toggle( false );
}

