#include "TePDIContrast.hpp"

#include <TePDIAgnostic.hpp>
#include <TePDITypes.hpp>
#include <TePDIUtils.hpp>
#include <TePDIHistogram.hpp>
#include <TePDIMatrix.hpp>

#include <math.h>
#include <float.h>


TePDIContrast::TePDIContrast()
{
}


TePDIContrast::~TePDIContrast()
{
}


bool TePDIContrast::RunImplementation()
{
  /* Getting general parameters */

  int contrast_type = 0;
  params_.GetParameter( "contrast_type", contrast_type );

  TePDITypes::TePDIRasterPtrType inRaster;
  int input_band = 0;
  if( params_.CheckParameter( "input_image", 
    TePDIParameters::TePDIRasterParam ) ) {
    
    params_.GetParameter( "input_image", inRaster );
    params_.GetParameter( "input_band", input_band );
  }

  int output_band = 0;
  TePDITypes::TePDIRasterPtrType outRaster;
  if( params_.CheckParameter( "output_image", 
    TePDIParameters::TePDIRasterParam ) ) {
    
    params_.GetParameter( "output_image", outRaster );
    params_.GetParameter( "output_band", output_band );  
  }  
  
  int histo_levels = 0;
  if( params_.CheckParameter( "histo_levels", 
      TePDIParameters::TePDIIntParam ) ) {
      
    params_.GetParameter( "histo_levels", histo_levels );
  }
  
  double min_level = 0;
  double max_level = 0;
  if( params_.CheckParameter( "min_level", 
    TePDIParameters::TePDIDoubleParam ) ) {
    
    params_.GetParameter( "min_level", min_level );
  }
  if( params_.CheckParameter( "max_level", 
    TePDIParameters::TePDIDoubleParam ) ) {
    
    params_.GetParameter( "max_level", max_level );
  }
  
  TePDIRgbPalette::pointer palette;
  if( params_.CheckParameter( "rgb_palette", 
    TePDIParameters::TePDIRgbPaletteParam ) ) {
  
    params_.GetParameter( "rgb_palette", palette );
  }
  
  /* Dumyy value definition */
 
  bool output_raster_uses_dummy = false;
  double output_raster_dummy = 0;
  
  if( outRaster.isActive() ) {
    output_raster_uses_dummy = outRaster->params().useDummy_;
    
    if( output_raster_uses_dummy ) {
      output_raster_dummy = outRaster->params().dummy_[ 0 ];
    }    
  }
  
  if( params_.CheckParameter( "dummy_value", 
    TePDIParameters::TePDIDoubleParam ) ) {
    
    params_.GetParameter( "dummy_value", output_raster_dummy );
    
    output_raster_uses_dummy = true;
  }  
  
  /* Reseting output raster */
  
  if( outRaster.isActive() ) {
    switch( contrast_type ) {
      case TePDIContrastSimpleSlicer :
      {
        if( ( ! params_.CheckParameter( "restrict_out_reset",
            TePDIParameters::TePDIIntParam ) ) ||
            ( inRaster->params().nlines_ != outRaster->params().nlines_ ) ||
            ( inRaster->params().ncols_ != outRaster->params().ncols_ ) ||
            ( output_band >= outRaster->params().nBands() ) ||
            ( 
              ( inRaster->projection() == outRaster->projection() ) ? 
              false : 
              ( 
                ( inRaster->projection() == 0 ) ?
                true :
                (
                  ( outRaster->projection() == 0 ) ? 
                  true :
                  ( inRaster->projection()->name() == 
                    outRaster->projection()->name() ) ? false : true
                )
              )  
            ) ||
            ( ! ( inRaster->params().box() == outRaster->params().box() ) ) ||
            ( 
              inRaster->params().photometric_[ 0 ] != 
              TeRasterParams::TePallete 
            ) ||
            ( outRaster->params().useDummy_ != output_raster_uses_dummy ) ||
            ( outRaster->params().useDummy_ ? 
              ( outRaster->params().dummy_[ 0 ] != output_raster_dummy ) :
              false
            )
          ) {
          
          TeRasterParams new_outRaster_params = outRaster->params();
          new_outRaster_params.nBands( output_band + 1 );
          if( inRaster->projection() == 0 ) {
            new_outRaster_params.projection( 0 );
          } else {
            new_outRaster_params.projection( 
              TeProjectionFactory::make( inRaster->projection()->params() ) );
          }
          new_outRaster_params.boxResolution( inRaster->params().box().x1(), 
            inRaster->params().box().y1(), inRaster->params().box().x2(), 
            inRaster->params().box().y2(), 
            inRaster->params().resx_, inRaster->params().resy_ );
            
          new_outRaster_params.setPhotometric( TeRasterParams::TePallete );
          
          new_outRaster_params.lutr_.clear();
          new_outRaster_params.lutg_.clear();
          new_outRaster_params.lutb_.clear();

          TePDIRgbPalette::iterator pal_it = palette->begin();
          TePDIRgbPalette::iterator pal_it_end = palette->end();
          for( unsigned int lut_index = 0 ; lut_index < palette->size() ;
              ++lut_index ) {
    
            new_outRaster_params.lutr_.push_back( 
              (unsigned short)pal_it->second.red_ );
            new_outRaster_params.lutg_.push_back( 
              (unsigned short)pal_it->second.green_ );
            new_outRaster_params.lutb_.push_back( 
              (unsigned short)pal_it->second.blue_ );

            ++pal_it;
          }
          
          if( output_raster_uses_dummy ) {
            new_outRaster_params.setDummy( output_raster_dummy, -1 );
          } else {
            new_outRaster_params.useDummy_ = false;
          }
            
          PDIAGN_TRUE_OR_RETURN( outRaster->init( new_outRaster_params ),            
            "Output raster reset error" );           
        }
      
        break;
      }  
      default :
      {
        /* Reseting output raster */
        
        if( ( ! params_.CheckParameter( "restrict_out_reset",
            TePDIParameters::TePDIIntParam ) ) ||
            ( inRaster->params().nlines_ != outRaster->params().nlines_ ) ||
            ( inRaster->params().ncols_ != outRaster->params().ncols_ ) ||
            ( output_band >= outRaster->params().nBands() ) ||
            ( 
              ( inRaster->projection() == outRaster->projection() ) ? 
              false : 
              ( 
                ( inRaster->projection() == 0 ) ?
                true :
                (
                  ( outRaster->projection() == 0 ) ? 
                  true :
                  ( inRaster->projection()->name() == 
                    outRaster->projection()->name() ) ? false : true
                )
              ) 
            ) ||
            ( ! ( inRaster->params().box() == outRaster->params().box() ) ) ||
            ( outRaster->params().useDummy_ != output_raster_uses_dummy ) ||
            ( outRaster->params().useDummy_ ? 
              ( outRaster->params().dummy_[ 0 ] != output_raster_dummy ) :
              false
            )
          ) {
        
          TeRasterParams new_outRaster_params = outRaster->params();
          new_outRaster_params.nBands( output_band + 1 );
          if( inRaster->projection() == 0 ) {
            new_outRaster_params.projection( 0 );
          } else {
            new_outRaster_params.projection( 
              TeProjectionFactory::make( inRaster->projection()->params() ) );
          }
          new_outRaster_params.boxResolution( inRaster->params().box().x1(), 
            inRaster->params().box().y1(), inRaster->params().box().x2(), 
            inRaster->params().box().y2(), 
            inRaster->params().resx_, inRaster->params().resy_ );            
            
          if( output_raster_uses_dummy ) {
            new_outRaster_params.setDummy( output_raster_dummy, -1 );
          } else {
            new_outRaster_params.useDummy_ = false;
          }            
            
          PDIAGN_TRUE_OR_RETURN( outRaster->init( new_outRaster_params ),
            "Output raster reset error" );             
        }
        
        break;
      }
    }
  }
  
  /* Generating histogram, if needed */
  
  TePDIHistogram::pointer histogram;
  if( params_.CheckParameter( "input_histogram", 
    TePDIParameters::TePDIHistogramPtrParam ) ) {
  
    params_.GetParameter( "input_histogram", histogram );
  } else {
    histogram.reset( new TePDIHistogram );
    PDIAGN_TRUE_OR_RETURN( 
      histogram->reset( inRaster, input_band, (unsigned int)histo_levels, 
      false ), "Histogram generation error" );
  }
  
  /* Getting channel range */
  
  double channel_min_level = DBL_MIN;
  double channel_max_level = DBL_MAX;
  
  if( outRaster.isActive() ) {
    PDIAGN_TRUE_OR_RETURN( TePDIUtils::TeGetRasterMinMaxBounds(
      outRaster, output_band, channel_min_level,
      channel_max_level ), "Unable to get raster channel level bounds" );  
  }
      
  /* Building a lut suitable for each algorithm */
    
  TePDITypes::TePDILutType lut;
  
  switch( contrast_type ) {
    case TePDIContrastMinMax :
    {
      lut = GetMinMaxLut( channel_min_level, channel_max_level,
        histogram );
  
      break;
    }
    case TePDIContrastLinear :
    {
      lut = GetLinearLut( histogram,
        min_level, max_level, channel_min_level, channel_max_level );
  
      break;
    }
    case TePDIContrastSquareRoot :
    {
      lut = GetSquareRootLut( histogram,
        min_level, max_level, channel_min_level, channel_max_level );
  
      break;
    }
    case TePDIContrastSquare :
    {
      lut = GetSquareLut( histogram,
        min_level, max_level, channel_min_level, channel_max_level );
  
      break;
    }
    case TePDIContrastLog :
    {
      lut = GetLogLut( histogram,
        min_level, max_level, channel_min_level, channel_max_level );
  
      break;
    }
    case TePDIContrastNegative :
    {
      lut = GetNegativeLut( histogram,
        min_level, max_level, channel_min_level, channel_max_level );
 
      break;
    }
    case TePDIContrastHistEqualizer :
    {
      lut = GetHistEqualizerLut( histogram,
        channel_min_level, channel_max_level );
  
      break;
    }
    case TePDIContrastSimpleSlicer :
    {
      GetSimpleSlicerLut( histogram,
        palette,  min_level, max_level, lut );
  
      break;
    }
    case TePDIContrastStat :
    {
      double target_mean = 0;
      params_.GetParameter( "target_mean", target_mean );
      
      double target_variance = 0;
      params_.GetParameter( "target_variance", target_variance );
      GetStatLut( histogram, target_mean, target_variance,
        channel_min_level, channel_max_level, lut );
  
      break;
    }    
    default :
    {
      PDIAGN_LOG_AND_THROW( "Unsuported contrast type" );
      break;
    }
  }
  
  /* Updating the output lut, if present */
  
  if( params_.CheckParameter( "outlut",
      TePDIParameters::TePDILutPtrParam ) ) {

    TePDITypes::TePDILutPtrType outlut;  
    params_.GetParameter( "outlut", outlut );
    
    *outlut = lut;
  }    

  /* Rendering output raster */
  
  if( outRaster.isActive() ) {
    RemapLevels( inRaster, lut, input_band, output_band, outRaster,
      output_raster_uses_dummy, output_raster_dummy, 
      histogram->hasFixedStep() );
  }
  
  return true;
}


bool TePDIContrast::CheckParameters( 
  const TePDIParameters& parameters ) const
{
  /* Checking input raster */

  if( parameters.CheckParameter( "input_image", 
    TePDIParameters::TePDIRasterParam ) ) {
    
    TePDITypes::TePDIRasterPtrType inRaster;
    
    if( ! parameters.GetParameter( "input_image", inRaster ) ) {
  
      PDIAGN_LOGERR( "Missing parameter: input_image" );
      return false;
    }
    if( ! inRaster.isActive() ) {
  
      PDIAGN_LOGERR( "Invalid parameter: input_image inactive" );
      return false;
    }
    if( inRaster->params().status_ == TeRasterParams::TeNotReady ) {
  
    PDIAGN_LOGERR( "Invalid parameter: input_image not ready" );
      return false;
    }
    
    /* Checking input band */
    
    int input_band;
    PDIAGN_TRUE_OR_RETURN( parameters.GetParameter( "input_band", input_band ),
      "Missing parameter: input_band" );
    PDIAGN_TRUE_OR_RETURN( ( ( input_band >= 0 ) && 
      ( input_band < inRaster->nBands() ) ),
      "Invalid parameter: input_band" );     
  }
        
  /* checking output raster, if present */
  
  TePDITypes::TePDIRasterPtrType output_image;
  if( parameters.CheckParameter( "output_image", 
    TePDIParameters::TePDIRasterParam ) ) {
    
    PDIAGN_TRUE_OR_RETURN( parameters.GetParameter( "output_image", 
      output_image ),  "Missing parameter: output_image" );
    PDIAGN_TRUE_OR_RETURN( output_image.isActive(),
      "Invalid parameter: output_image inactive" );
    PDIAGN_TRUE_OR_RETURN( output_image->params().status_ != 
      TeRasterParams::TeNotReady, 
      "Invalid parameter: output_image not ready" );
      
    /* Checking output_band */
    
    int output_band;
    PDIAGN_TRUE_OR_RETURN( parameters.GetParameter( "output_band", 
      output_band ), "Missing parameter: output_band" );
    PDIAGN_TRUE_OR_RETURN( ( output_band >= 0 ),
      "Invalid parameter: output_band" ); 
      
    /* Input raster needed if output_image is present */
    
    PDIAGN_TRUE_OR_RETURN( parameters.CheckParameter( "input_image", 
      TePDIParameters::TePDIRasterParam ), 
      "Missing parameter: input_image" );
  }

  /* Checking for the correct allowed contrast types */

  int contrast_type;
  if( ! parameters.GetParameter( "contrast_type", contrast_type ) ) {
    PDIAGN_LOGERR( "Missing parameter: contrast_type" );
    return false;
  }
  PDIAGN_CHECK_NOTEQUAL( contrast_type, 0, "Invalid Contrast type" );
  if( ( contrast_type != TePDIContrastMinMax ) &&
      ( contrast_type != TePDIContrastLinear ) &&
      ( contrast_type != TePDIContrastSquareRoot ) &&
      ( contrast_type != TePDIContrastSquare ) &&
      ( contrast_type != TePDIContrastLog ) &&
      ( contrast_type != TePDIContrastNegative ) &&
      ( contrast_type != TePDIContrastHistEqualizer ) &&
      ( contrast_type != TePDIContrastSimpleSlicer ) &&
      ( contrast_type != TePDIContrastStat ) ) {

    PDIAGN_LOGERR( "Invalid parameter: contrast_type" );
    return false;
  }

  /* Checking for min and max required parameters */

  if( ( contrast_type == TePDIContrastLinear ) ||
      ( contrast_type == TePDIContrastSquareRoot ) ||
      ( contrast_type == TePDIContrastSquare ) ||
      ( contrast_type == TePDIContrastLog ) ||
      ( contrast_type == TePDIContrastNegative ) ||
      ( contrast_type == TePDIContrastSimpleSlicer ) ) {

    if( ! parameters.CheckParameter( "min_level",
          TePDIParameters::TePDIDoubleParam ) ) {

      PDIAGN_LOGERR( "Missing parameter: min_level" );
      return false;
    }
    if( ! parameters.CheckParameter( "max_level",
          TePDIParameters::TePDIDoubleParam ) ) {

      PDIAGN_LOGERR( "Missing parameter: max_level" );
      return false;
    }
  }

  /* Checking for RGB Palette required parameters */

  if( ( contrast_type == TePDIContrastSimpleSlicer ) ) {
    TePDIRgbPalette::pointer rgb_palette;

    if( ( ! parameters.GetParameter( "rgb_palette", rgb_palette ) ) ||
        ( ! rgb_palette.isActive() ) ) {

      PDIAGN_LOGERR( "Missing parameter: rgb_palette" );
      return false;
    }
  }
  
  /* checking outlut parameter */
  
  if( parameters.CheckParameter( "outlut",
      TePDIParameters::TePDILutPtrParam ) ) {

    TePDITypes::TePDILutPtrType outlut;  
    parameters.GetParameter( "outlut", outlut );
    
    PDIAGN_TRUE_OR_RETURN( outlut.isActive(),
      "Invalid parameter: outlut" );
  }
  
  /* checking input_histogram */
  
  TePDIHistogram::pointer input_histogram;
  
  if( parameters.CheckParameter( "input_histogram", 
    TePDIParameters::TePDIHistogramPtrParam ) ) {
  
    parameters.GetParameter( "input_histogram", input_histogram );
    PDIAGN_TRUE_OR_RETURN( input_histogram.isActive(),
      "Invalid parameter: input_histogram" );
    PDIAGN_TRUE_OR_RETURN( ( input_histogram->size() > 0 ),
      "Invalid parameter: input_histogram" );
  } else {
    /* Input raster needed if input_histogram isn't present */
    
    PDIAGN_TRUE_OR_RETURN( parameters.CheckParameter( "input_image", 
      TePDIParameters::TePDIRasterParam ), 
      "Missing parameter: input_image" );
  }
  
  /* Checking target_mean and target_variance */
  
  if( ( contrast_type == TePDIContrastStat ) ) {
    double target_mean = 0;
    PDIAGN_TRUE_OR_RETURN( 
      parameters.GetParameter( "target_mean", target_mean ),
      "Missing parameter: target_mean" );
      
    double target_variance = 0;
    PDIAGN_TRUE_OR_RETURN( 
      parameters.GetParameter( "target_variance", target_variance ),
      "Missing parameter: target_variance" );      
  }
  
  return true;
}


void TePDIContrast::ResetState( const TePDIParameters& )
{
}


TePDITypes::TePDILutType TePDIContrast::GetMinMaxLut(
  double channel_min_level, double channel_max_level,
  TePDIHistogram::pointer hist )
{
  PDIAGN_TRUE_OR_THROW( hist->size() != 0, "Invalid histogram" );
  
  double histo_max = DBL_MIN;
  double histo_min = DBL_MAX;
  
  TePDIHistogram::iterator it = hist->begin();
  TePDIHistogram::iterator it_end = hist->end();
  
  while( it != it_end ) {
    if( it->first < histo_min ) {
      histo_min = it->first;
    }
    if( it->first > histo_max ) {
      histo_max = it->first;
    }
  
    ++it;
  }

  return GetLinearLut( hist, histo_min,
    histo_max, channel_min_level, channel_max_level );
}


TePDITypes::TePDILutType TePDIContrast::GetLinearLut(
  TePDIHistogram::pointer hist,
  double min, double max,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( max > min, "Invalid max and min values" );

  /* Calculating parameters */

  unsigned int levels = hist->size();

  double a = 0;
  double b = 0;

  if( max == min ) {
    a = (double)levels;
    b = -1. * ((double)levels) * min;
  } else {
    a = ((double)levels) / ( max - min );
    b = ( -1. * ((double)levels) * min ) / ( max - min );
  }

  /* Generating LUT map using the existing histogram levels */

  TePDITypes::TePDILutType out_lut;

  TePDIHistogram::iterator hist_it = hist->begin();
  TePDIHistogram::iterator hist_it_end = hist->end();
  
  unsigned int progress = 0;
  double mapped_level = 0;
  StartProgInt( "Building Linear Lut...", hist->size() );

  while( hist_it != hist_it_end ) {
    UpdateProgInt( progress );
    
    if( hist_it->first <= min ) {
      out_lut[ hist_it->first ] = channel_min_level;
    } else if( hist_it->first >= max ) {
      out_lut[ hist_it->first ] = channel_max_level;
    } else {
      mapped_level = ( a * hist_it->first ) + b;
      
      if( mapped_level < channel_min_level ) {
        mapped_level = channel_min_level;
      } else if( mapped_level > channel_max_level ) {
        mapped_level = channel_max_level;
      }
      
      out_lut[ hist_it->first ] = mapped_level;
    }

    ++progress;
    ++hist_it;
  }

  return out_lut;
}


TePDITypes::TePDILutType TePDIContrast::GetSquareRootLut(
  TePDIHistogram::pointer hist,
  double min, double max,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( min < max, "Invalid min and max values" );

  unsigned int levels = hist->size();

  double factor = ((double)levels) / sqrt( max - min );

  /* Generating LUT map using the existing histogram levels */

  TePDITypes::TePDILutType out_lut;

  TePDIHistogram::iterator hist_it = hist->begin();
  TePDIHistogram::iterator hist_it_end = hist->end();
  
  unsigned int progress = 0;
  double mapped_level = 0;
  StartProgInt( "Building Square Root Lut...", hist->size() );  

  while( hist_it != hist_it_end ) {
    UpdateProgInt( progress );
    
    if( hist_it->first <= min ) {
      out_lut[ hist_it->first ] = channel_min_level;
    } else if( hist_it->first >= max ) {
      out_lut[ hist_it->first ] = channel_max_level;
    } else {
      mapped_level = factor * sqrt( hist_it->first - min );
      
      if( mapped_level < channel_min_level ) {
        mapped_level = channel_min_level;
      } else if( mapped_level > channel_max_level ) {
        mapped_level = channel_max_level;
      }      
      
      out_lut[ hist_it->first ] = mapped_level;
    }

    ++progress;
    ++hist_it;
  }

  return out_lut;
}


TePDITypes::TePDILutType TePDIContrast::GetSquareLut(
  TePDIHistogram::pointer hist,
  double min, double max,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( min < max, "Invalid min and max values" );

  unsigned int levels = hist->size();

  double factor = ((double)levels) / pow( (max - min), 2 );

  /* Generating LUT map using the existing histogram levels */

  TePDITypes::TePDILutType out_lut;

  TePDIHistogram::iterator hist_it = hist->begin();
  TePDIHistogram::iterator hist_it_end = hist->end();

  unsigned int progress = 0;
  double mapped_level = 0;
  StartProgInt( "Building Square Lut...", hist->size() ); 
    
  while( hist_it != hist_it_end ) {
    UpdateProgInt( progress );
    
    if( hist_it->first <= min ) {
      out_lut[ hist_it->first ] = channel_min_level;
    } else if( hist_it->first >= max ) {
      out_lut[ hist_it->first ] = channel_max_level;
    } else {
      mapped_level = factor * pow( hist_it->first - min, 2 );
      
      if( mapped_level < channel_min_level ) {
        mapped_level = channel_min_level;
      } else if( mapped_level > channel_max_level ) {
        mapped_level = channel_max_level;
      }        
      
      out_lut[ hist_it->first ] = mapped_level;
    }

    ++progress;
    ++hist_it;
  }

  return out_lut;
}


TePDITypes::TePDILutType TePDIContrast::GetLogLut(
  TePDIHistogram::pointer hist,
  double min, double max,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( max > ( min+1 ), "Invalid min and max values" );

  unsigned int levels = hist->size();

  double factor = ((double)levels) / log10( max - min + 1 );

  /* Generating LUT map using the existing histogram levels */

  TePDITypes::TePDILutType out_lut;

  TePDIHistogram::iterator hist_it = hist->begin();
  TePDIHistogram::iterator hist_it_end = hist->end();

  unsigned int progress = 0;
  double mapped_level = 0;
  StartProgInt( "Building Log Lut...", hist->size() ); 
    
  while( hist_it != hist_it_end ) {
    UpdateProgInt( progress );
    
    if( hist_it->first <= min ) {
      out_lut[ hist_it->first ] = channel_min_level;
    } else if( hist_it->first >= max ) {
      out_lut[ hist_it->first ] = channel_max_level;
    } else {
      mapped_level = factor * log10( hist_it->first - min + 1 );
      
      if( mapped_level < channel_min_level ) {
        mapped_level = channel_min_level;
      } else if( mapped_level > channel_max_level ) {
        mapped_level = channel_max_level;
      }        
    
      out_lut[ hist_it->first ] = mapped_level;
    }

    ++progress;
    ++hist_it;
  }

  return out_lut;
}


TePDITypes::TePDILutType TePDIContrast::GetNegativeLut(
  TePDIHistogram::pointer hist,
  double min, double max,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( max > min, "Invalid max and min values" );

  /* Calculating parameters */

  unsigned int levels = hist->size();

  double a = -1. * ((double)levels) / ( max - min );
  double b = ( ((double)levels) * max ) / ( max - min );

  /* Generating LUT map using the existing histogram levels */

  TePDITypes::TePDILutType out_lut;

  TePDIHistogram::iterator hist_it = hist->begin();
  TePDIHistogram::iterator hist_it_end = hist->end();

  unsigned int progress = 0;
  double mapped_level = 0;
  StartProgInt( "Building Negative Lut...", hist->size() ); 
    
  while( hist_it != hist_it_end ) {
    UpdateProgInt( progress );
    
    if( hist_it->first <= min ) {
      out_lut[ hist_it->first ] = channel_max_level;
    } else if( hist_it->first >= max ) {
      out_lut[ hist_it->first ] = channel_min_level;
    } else {
      mapped_level = ( a * hist_it->first ) + b;
      
      if( mapped_level < channel_min_level ) {
        mapped_level = channel_min_level;
      } else if( mapped_level > channel_max_level ) {
        mapped_level = channel_max_level;
      }
      
      out_lut[ hist_it->first ] = mapped_level;
    }

    ++progress;
    ++hist_it;
  }

  return out_lut;
}

TePDITypes::TePDILutType TePDIContrast::GetHistEqualizerLut(
  TePDIHistogram::pointer hist,
  double channel_min_level, double channel_max_level )
{
  PDIAGN_TRUE_OR_THROW( hist->size() > 1, "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( ( channel_max_level > channel_min_level ),
    "Invalid paramters channel_max_level <= channel_min_level" );
  PDIAGN_TRUE_OR_THROW( ( hist->GetMaxCount() > 0 ),
    "Invalid histogram" );
    
  /* Generating the accumulative matrix */

  TePDIHistogram::iterator in_hist_it = hist->begin();
  TePDIHistogram::iterator in_hist_it_end = hist->end();

  TePDIMatrix< double > accumulative_matrix;
  PDIAGN_TRUE_OR_THROW( accumulative_matrix.Reset( hist->size(), 2 ),
    "Matrix reset error" );
    
  accumulative_matrix( 0, 0 ) = in_hist_it->first;
  accumulative_matrix( 0, 1 ) = (double)in_hist_it->second;
  ++in_hist_it;
  
  unsigned int accumulative_matrix_line = 1;
  double hist_population = (double)in_hist_it->second;
  
  StartProgInt( "Building Histogram Equalizer Lut...", 2 * hist->size() );
  
  while( in_hist_it != in_hist_it_end ) { 
    accumulative_matrix( accumulative_matrix_line, 0 ) = in_hist_it->first;
    accumulative_matrix( accumulative_matrix_line, 1 ) =
      accumulative_matrix( accumulative_matrix_line - 1, 1 ) +
      (double)in_hist_it->second;
      
     hist_population += (double)in_hist_it->second;
  
    ++accumulative_matrix_line;
    ++in_hist_it;
    
    IncProgInt();
  }
  
  /* Creating the look-up table */

  double total_levels_nmb = (double)hist->size();
  double mapped_level = 0;
  TePDITypes::TePDILutType out_lut;
  
  for( accumulative_matrix_line = 0 ; 
    accumulative_matrix_line < accumulative_matrix.GetLines() ;
    ++accumulative_matrix_line ) {
    
    mapped_level = ( accumulative_matrix( accumulative_matrix_line, 1 ) *
      total_levels_nmb ) / hist_population;
      
    if( mapped_level < channel_min_level ) {
      mapped_level = channel_min_level;
    } else if( mapped_level > channel_max_level ) {
      mapped_level = channel_max_level;
    }
    
    out_lut[ accumulative_matrix( accumulative_matrix_line, 0 ) ] = 
      mapped_level;
      
    IncProgInt();  
  }
  
  StopProgInt();
  
  return out_lut;
}


void TePDIContrast::GetSimpleSlicerLut(
  TePDIHistogram::pointer hist,
  TePDIRgbPalette::pointer in_palette,
  double min,
  double max,
  TePDITypes::TePDILutType& out_lut )
{
  PDIAGN_TRUE_OR_THROW( ( hist->size() > 0 ), "Invalid histogram size" );
  PDIAGN_TRUE_OR_THROW( max > min, "Invalid max and min values" );
  PDIAGN_TRUE_OR_THROW( in_palette->size() > 0,
    "Invalid input palette size" );

  out_lut.clear();

  /* Extracting palette levels */

  std::vector< double > palette_levels;
  TePDIRgbPalette::iterator pal_it = in_palette->begin();
  TePDIRgbPalette::iterator pal_it_end = in_palette->end();
  
  unsigned int progress = 0;
  StartProgInt( "Building Simple Slicer Lut...", hist->size() +
    in_palette->size() );    

  while( pal_it != pal_it_end ) {
    UpdateProgInt( progress );
    
    palette_levels.push_back( pal_it->first );

    ++progress;
    ++pal_it;
  }

  /* min and max adjusting to the supplied histogram range */

  double in_hist_max = hist->GetMaxLevel();
  double in_hist_min = hist->GetMinLevel();

  min = ( min < in_hist_min ) ? in_hist_min : min;
  max = ( max > in_hist_max ) ? in_hist_max : max;

  /* Output LUT generation */

  double slice_size = ( max - min ) / ((double)in_palette->size());

  double first_pal_level = palette_levels[ 0 ];
  double last_pal_level = palette_levels[ palette_levels.size() - 1 ];

  TePDIHistogram::iterator in_hist_it = hist->begin();
  TePDIHistogram::iterator in_hist_it_end = hist->end();

  double current_level;
  unsigned int current_slice;
  
  while( in_hist_it != in_hist_it_end ) {
    UpdateProgInt( progress );
    
    current_level = in_hist_it->first;

    if( current_level < min ) {
      out_lut[ current_level ] = first_pal_level;
    } else if ( current_level > max ) {
      out_lut[ current_level ] = last_pal_level;
    } else {
      current_slice =
        (unsigned int) floor( ( current_level - min ) / slice_size );
        
      PDIAGN_DEBUG_CONDITION( ( current_slice < palette_levels.size() ),
        "Invalid current_slice=" + Te2String( current_slice ) +
        " for pallete_levels size=" + Te2String( palette_levels.size() ) );

      out_lut[ current_level ] = palette_levels[ current_slice ];
    }

    ++progress;
    ++in_hist_it;
  }


}


void TePDIContrast::GetStatLut( TePDIHistogram::pointer hist,
  double target_mean, double target_variance, double channel_min_level, 
  double channel_max_level, TePDITypes::TePDILutType& out_lut )
{
  PDIAGN_CHECK_NOTEQUAL( hist->size(), 0, "Invalid histogram size" );

  out_lut.clear();
  
  TePDIHistogram::iterator in_hist_it;
  TePDIHistogram::iterator in_hist_it_end = hist->end(); 
  
  /* Calculating the current mean */ 
  
  double current_mean = 0;
  unsigned int total_pixels = 0;
    
  in_hist_it = hist->begin();
  while( in_hist_it != in_hist_it_end ) {
    current_mean += ( in_hist_it->first * ((double)in_hist_it->second) );
    total_pixels += in_hist_it->second;
      
    ++in_hist_it;
  }
  
  current_mean = current_mean / ((double)total_pixels);
    
  /* Calculating the current variance */ 
        
  double current_variance = 0;

  double temp_double = 0;
  
  in_hist_it = hist->begin();
  while( in_hist_it != in_hist_it_end ) {
    temp_double = ( in_hist_it->first - current_mean );
    temp_double = temp_double * temp_double * ((double)in_hist_it->second);
    
    current_variance += temp_double;
      
    ++in_hist_it;
  }
  current_variance = current_variance / ( (double) total_pixels );
  
  /* Creating levels map */
  
  double gain = sqrt( target_variance / current_variance );
  double offset = target_mean - ( gain * current_mean );
  
  in_hist_it = hist->begin();
  
  double current_level = 0;
  double mapped_level = 0;
  
  while( in_hist_it != in_hist_it_end ) {
    current_level = in_hist_it->first;
    mapped_level = ( current_level * gain ) + offset;

    if( mapped_level < channel_min_level ) {
      out_lut[ current_level ] = channel_min_level;
    } else if ( mapped_level > channel_max_level ) {
      out_lut[ current_level ] = channel_max_level;
    } else {
      out_lut[ current_level ] = mapped_level;
    }    
  
    ++in_hist_it;
  }
}


void TePDIContrast::RemapLevels(
  TePDITypes::TePDIRasterPtrType& inRaster,
  TePDITypes::TePDILutType& lut,
  int in_channel,
  int out_channel,
  TePDITypes::TePDIRasterPtrType& outRaster,
  bool use_dummy, double dummy_value, bool fixed_step_lut )
{
  PDIAGN_TRUE_OR_THROW( inRaster.isActive(),
    "inRaster inactive" );
  PDIAGN_TRUE_OR_THROW( outRaster.isActive(),
    "outRaster inactive" );
  PDIAGN_TRUE_OR_THROW( 
    inRaster->params().status_ != TeRasterParams::TeNotReady,
    "inRaster not ready" );
  PDIAGN_TRUE_OR_THROW( 
    ( outRaster->params().status_ != TeRasterParams::TeNotReady ),
    "outRaster not ready" );
  PDIAGN_CHECK_EQUAL( inRaster->params().nlines_,
    outRaster->params().nlines_,
    "Lines number mismatch between input and output image" );
  PDIAGN_CHECK_EQUAL( inRaster->params().ncols_,
    outRaster->params().ncols_,
    "Columns number mismatch between input and output image" );
  PDIAGN_TRUE_OR_THROW( in_channel < inRaster->nBands(), 
    "Invalid input band" );
  PDIAGN_TRUE_OR_THROW( out_channel < outRaster->nBands(), 
    "Invalid output band" );
  PDIAGN_TRUE_OR_THROW( ( lut.size() > 1 ), "Invalid lut" );

  const int raster_lines = inRaster->params().nlines_;
  const int raster_columns = inRaster->params().ncols_;
  
  /* Guessing dummy use */
  
  bool inRaster_uses_dummy = inRaster->params().useDummy_;
  
  bool outRaster_uses_dummy = outRaster->params().useDummy_;
  double outRaster_dummy = 0;
  if( outRaster_uses_dummy ) {
    outRaster_dummy = outRaster->params().dummy_[ out_channel ];
  } else {
    outRaster_dummy = dummy_value;
  }
  
  /* Loading lut */
  
  TePDIMatrix< double > lutmatrix;
  PDIAGN_TRUE_OR_THROW( lutmatrix.Reset( 2, lut.size(), 
    TePDIMatrix< double >::AutoMemPol ),
    "Unable to create lut matrix" );
  {
    TePDITypes::TePDILutType::iterator it = lut.begin();;
  
    for( unsigned int lutcol = 0 ; lutcol < lutmatrix.GetColumns() ; 
      ++lutcol ) {
      
      lutmatrix( 0, lutcol ) = it->first;
      lutmatrix( 1, lutcol ) = it->second;
      
      ++it;
    }
  }
  
  /* Remapping levels */
  
  StartProgInt( "Remapping Levels...", raster_lines );

  if( fixed_step_lut ) {
    double current_level = 0;
    const double lut_min_level = lutmatrix( 0, 0 );
    const double lut_max_level = lutmatrix( 0, lutmatrix.GetColumns() - 1 );
    unsigned int best_lut_index = 0;
    const unsigned int lutmatrix_last_index = ( lutmatrix.GetColumns() - 1 );
    
    double lut_step = 0;
    if( lutmatrix.GetColumns() > 1 ) {
      lut_step = lutmatrix( 0, 1 ) - lutmatrix( 0, 0 );
    }
  
    for( int line = 0 ; line < raster_lines ; ++line ) {
      UpdateProgInt( line );
    
      for( int column = 0 ; column < raster_columns ; ++column ) {
        if( inRaster->getElement( column, line, current_level,
            in_channel ) ) {
            
          if( use_dummy && ( current_level == dummy_value ) ) {
            PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
              outRaster_dummy, out_channel ),
              "Level remmaping error at " + Te2String( line ) +
              "," + Te2String( column ) );           
          } else {
            /* Finding the mapping level from lut */
            
            if( current_level < lut_min_level ) {
              best_lut_index = 0;
            } else if( current_level > lut_max_level ) {
              best_lut_index = lutmatrix_last_index;
            } else {
              best_lut_index = ( unsigned int ) ( TeRound( ( current_level - 
                lut_min_level ) / lut_step ) );             
            }
            
            /* Pixel Output level remapping */
            
            PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
              lutmatrix( 1, best_lut_index ), out_channel ),
              "Level remmaping error at " + Te2String( line ) +
              "," + Te2String( column ) );
          }
        } else {
          PDIAGN_TRUE_OR_THROW( inRaster_uses_dummy, "Raster read error" );
            
          PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
            outRaster_dummy, out_channel ),
            "Level remmaping error at " + Te2String( line ) +
            "," + Te2String( column ) );          
        }
      }
    }
  } else {
    double current_level;
    unsigned int left_element_index;
    unsigned int middle_element_index;
    unsigned int right_element_index;
    const unsigned int last_valid_index = lutmatrix.GetColumns() - 1;
    const unsigned int lut_size = lutmatrix.GetColumns();
    double middle_element_level;
  
    for( int line = 0 ; line < raster_lines ; ++line ) {
      UpdateProgInt( line );
    
      for( int column = 0 ; column < raster_columns ; ++column ) {
        if( inRaster->getElement( column, line, current_level,
            in_channel ) ) {
            
          if( use_dummy && ( current_level == dummy_value ) ) {
            PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
              outRaster_dummy, out_channel ),
              "Level remmaping error at " + Te2String( line ) +
              "," + Te2String( column ) );           
          } else {            
            /* Finding the two best mapping levels from lut */
            
            if( lut_size == 1 ) {
              right_element_index = left_element_index = 1;
            } else {
              left_element_index = 0;
              right_element_index = last_valid_index;
            
              do {
                middle_element_index =  left_element_index + 
                  ( ( right_element_index - left_element_index ) / 2 );
                middle_element_level = lutmatrix( 0, middle_element_index );
                          
                if( current_level == middle_element_level ) {
                  left_element_index = right_element_index = 
                    middle_element_index;
                  break;
                } else if( current_level < middle_element_level ) {
                  right_element_index = middle_element_index;
                } else {
                  left_element_index = middle_element_index;
                }          
              } while( ( right_element_index - left_element_index ) > 1 );
            }
            
            /* Pixel Output level aproximation and mapping */
            
            if( ( current_level - lutmatrix( 0, left_element_index ) ) <
                ( lutmatrix( 0, right_element_index ) - current_level ) ) {
    
              PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
                lutmatrix( 1, left_element_index ), out_channel ),
                "Level remmaping error at " + Te2String( line ) +
                "," + Te2String( column ) );
            } else {
              PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
                lutmatrix( 1, right_element_index ), out_channel ),
                "Level remmaping error at " + Te2String( line ) +
                "," + Te2String( column ) );
            }
          }
        } else {
          PDIAGN_TRUE_OR_THROW( inRaster_uses_dummy, "Raster read error" );
          
          PDIAGN_TRUE_OR_THROW( outRaster->setElement( column, line,
            outRaster_dummy, out_channel ),
            "Level remmaping error at " + Te2String( line ) +
            "," + Te2String( column ) );          
        }
      }
    }
  }
  
  StopProgInt();
}

