#include "TePDIMappedMemory.hpp"
#include "TePDIUtils.hpp"

#if defined WIN32
  #include <windows.h>
#elif defined __unix__
  #include <sys/types.h>
  #include <sys/stat.h>
  #include <fcntl.h>
  #include <unistd.h>
#else
  #error "Unsuported plataform"
#endif


void TePDIMappedMemory::Init()
{
  #if defined WIN32
    m_hFile_ = 0;
    m_hMapping_ = 0;
    m_lpszFile_ = 0;
    m_dwSize_ = 0;
  #elif defined __unix__
    m_hFile_ = 0;
    m_lpszFile_ = 0;
    m_dwSize_ = 0;
  #else
    #error "Unsuported plataform"
  #endif
  
  disk_file_name_.clear();
  mapping_is_active_ = false;
};


TePDIMappedMemory::TePDIMappedMemory( unsigned long int size )
{
  Init();
  Reset( size );
}


TePDIMappedMemory::~TePDIMappedMemory()
{
  Reset( 0 );
}


bool TePDIMappedMemory::Reset( unsigned long int size )
{
  if( size != ((unsigned long int)m_dwSize_) ) {
    if( mapping_is_active_ ) {
      #if defined WIN32
        PDIAGN_TRUE_OR_LOG( UnmapViewOfFile( m_lpszFile_ ),
          "Unable to unmap disk file" );
        
        CloseHandle( m_hMapping_ );
        CloseHandle( m_hFile_ );
      #elif defined __unix__
        PDIAGN_TRUE_OR_LOG( ( munmap( m_lpszFile_ , m_dwSize_ ) != -1),
          "Unable to unmap disk file" );
          
        close(m_hFile_);
      #else
        #error "Unsuported plataform"
      #endif
      
      remove( disk_file_name_.c_str() );
      
      Init();
    }
    
    if( size > 0 ) {
      std::string disk_file_name;
    
      PDIAGN_TRUE_OR_RETURN( TeGetTempFileName( disk_file_name ),
        "Unable to get temporary file name" );
    
      #if defined WIN32
        DWORD dwCreationDisposition = OPEN_ALWAYS;
        DWORD dwDesiredAccess = GENERIC_READ | GENERIC_WRITE;         
        DWORD flProtect = PAGE_READWRITE;
        DWORD dwDesiredAccessV = FILE_MAP_WRITE; 
        
        HANDLE m_hFile = CreateFileA(
          disk_file_name.c_str(), dwDesiredAccess, 
          FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, dwCreationDisposition,
          0, NULL);
        PDIAGN_TRUE_OR_RETURN( ( m_hFile != INVALID_HANDLE_VALUE ),
          "Temporary file creation error" );
          
        DWORD m_dwSize = GetFileSize(m_hFile, NULL);
        PDIAGN_TRUE_OR_RETURN( ( m_dwSize != 0xffffffff ),
          "Invalid file size" );
          
        if ( m_dwSize < size ) {
          char lpBuffer[1024];
          int nNumberOfBytesToWrite = size - m_dwSize;
          unsigned long nNumberOfBytesWritten;
          
          for (;nNumberOfBytesToWrite > 1024; nNumberOfBytesToWrite-=1024) {
            if ( ! WriteFile( m_hFile, lpBuffer, 1024, &nNumberOfBytesWritten, 
                 NULL ) ) {
                 
              CloseHandle(m_hFile);
              
              PDIAGN_LOG_AND_RETURN( "Unable to write to temporary file" );
            }
          }
          
          if ( ! WriteFile( m_hFile, lpBuffer, nNumberOfBytesToWrite, 
               &nNumberOfBytesWritten, NULL ) ) {
               
            CloseHandle(m_hFile);
            
            PDIAGN_LOG_AND_RETURN( "Unable to write to temporary file" );
          }       
        }
        
        HANDLE m_hMapping = CreateFileMapping( m_hFile, NULL, flProtect,
          0, 0, NULL );
                
        PDIAGN_TRUE_OR_RETURN( ( m_hMapping != NULL ),
          "Mapping creation error" );
          
        LPVOID m_lpszFile = (LPVOID) MapViewOfFile( m_hMapping, 
          dwDesiredAccessV, 0, 0, 0);
        
        PDIAGN_TRUE_OR_RETURN( ( m_lpszFile != 0 ), "Mapping creation error" );        
        
        m_hFile_ = m_hFile;
        m_hMapping_ = m_hMapping;
        m_lpszFile_ = m_lpszFile;
        m_dwSize_ = m_dwSize;
      #elif defined __unix__
        long m_dwSize = (long)size;    
        
        int m_hFile = open( disk_file_name.c_str(),O_RDWR|O_CREAT,S_IRWXU);
        PDIAGN_TRUE_OR_RETURN( ( m_hFile != -1 ), 
          "Temporary file creation error" );
          
        PDIAGN_TRUE_OR_RETURN( ( lseek(m_hFile, m_dwSize - 1, SEEK_SET) != -1 ),
          "Temporary file seek error" );
          
        unsigned char c = '\0';
        
        PDIAGN_TRUE_OR_RETURN( 
          ( write( m_hFile, (void*)&c, sizeof( unsigned char ) ) != -1 ),
          "Temporary file write error" );          
        
        void* m_lpszFile  =  mmap(0, m_dwSize, (PROT_READ | PROT_WRITE), 
          MAP_SHARED, m_hFile, 0);
        PDIAGN_TRUE_OR_RETURN( ( m_lpszFile != ((void*)-1) ),
          "Mapping creation error" );
          
        m_hFile_ = m_hFile;
        m_lpszFile_ = m_lpszFile;
        m_dwSize_ = m_dwSize;
      #else
        #error "Unsuported plataform"
      #endif
      
      disk_file_name_ = disk_file_name;
      mapping_is_active_ = true;      
    }
  }
  
  return true;
}

