#include "TePDIPIManager.hpp"

#include "TePDIAgnostic.hpp"

#include <TeProgress.h>
#include <math.h>


TePDIPIManager::TePDIPIManager( const std::string& message, 
  unsigned long int tsteps, bool enabled )
{
  Reset( message, tsteps );
  Toggle( enabled );
}


TePDIPIManager::~TePDIPIManager()
{
  Toggle( false );
};


void TePDIPIManager::Toggle( bool enabled )
{
  if( ( ! my_settings_.enabled ) && enabled ) {
    TePDIPIManagerSettings& global_settings = GetGlobalSettings();
    old_global_settings_ = global_settings;
    
    TeProgressBase* prog_intef = TeProgress::instance();
    
    if( ( ! old_global_settings_.enabled ) && prog_intef ) {
      prog_intef->reset();
      prog_intef->setProgress( 0 );
    }
  
    UpdateProgressInterface( my_settings_ );
    global_settings = my_settings_;
    
    my_settings_.enabled = true;
  } if( my_settings_.enabled && ( ! enabled ) ) {
    TePDIPIManagerSettings& global_settings = GetGlobalSettings();
    global_settings = old_global_settings_;
    
    if( global_settings.enabled ) {
      UpdateProgressInterface( global_settings );
    } else {
      TeProgressBase* prog_intef = TeProgress::instance();
      
      if( prog_intef ) {
        prog_intef->reset();
        prog_intef->cancel();
      }
    }
    
    my_settings_.enabled = false;
  }
}


void TePDIPIManager::UpdateProgressInterface( 
  const TePDIPIManagerSettings& settings ) const
{
  TeProgressBase* prog_intef = TeProgress::instance();
  
  if( prog_intef && settings.enabled ) {
    prog_intef->setMessage( settings.message );
    prog_intef->setTotalSteps( 100 );
    int proportional_progress = 
      (int)( 100. * ( ( (double)settings.currstep ) / 
      ( (double)settings.tsteps ) ) );
    prog_intef->setProgress( proportional_progress );
  }
}


void TePDIPIManager::Update( unsigned long int step )
{
  if( step <= my_settings_.tsteps ) {
    my_settings_.currstep = step;
    
    if( step >= my_settings_.nxtblk_index ) {
      UpdateProgressInterface( my_settings_ );
      my_settings_.nxtblk_index += my_settings_.blksize;
    }
  } else {
    if( my_settings_.currstep < my_settings_.tsteps ) {
      my_settings_.currstep = my_settings_.tsteps;
      my_settings_.nxtblk_index += my_settings_.blksize;
      UpdateProgressInterface( my_settings_ );
    }
  }
}


void TePDIPIManager::Increment()
{
  Update( my_settings_.currstep + 1 );
}


void TePDIPIManager::Reset( const std::string& message, 
  unsigned long int tsteps )
{
  Toggle( false );

  TePDIPIManagerSettings empty_settings;
  my_settings_ = empty_settings;
  
  my_settings_.tsteps = tsteps; 
  my_settings_.message = message;
  
  if( my_settings_.tsteps > 20 ) {
    my_settings_.blksize = 
      ( unsigned long int ) ceil( ((double)my_settings_.tsteps) / 20.0 );
  } else {
    my_settings_.blksize = 1;
  }
  
  my_settings_.nxtblk_index = my_settings_.blksize;
  
  if( tsteps > 0 ) {
    Toggle( true );
  }
}

