#include "TePDIParameters.hpp"

#include <TePDIAgnostic.hpp>

#include <TeUtils.h>

TePDIParameters::TePDIParameters()
{
  last_up_time_ = 0;
}


TePDIParameters::~TePDIParameters()
{

}


void TePDIParameters::Clear()
{
  RasterMap_.clear();
  RasterVecMap_.clear();
  RasterVecPtrMap_.clear();
  IntegerMap_.clear();
  IntegerVectorMap_.clear();
  DoubleMap_.clear();
  DoubleVectorMap_.clear();
  StringMap_.clear();
  PDIBoxMap_.clear();
  PDIRgbPaletteMap_.clear();
  PDIFilterMaskMap_.clear();
  TePolygonMap_.clear();
  TePDILutMap_.clear();
  TePDILutPtrMap_.clear();
  TePDILayerMap_.clear();
  TePolygonSetMap_.clear();
  TePolygonSetPtrMap_.clear();
  TePDIHistogramPtrMap_.clear();
  TeCoordPairVectPtrMap_.clear();
  hist_vect_map_.clear();
  TePDIPolSetMapPtrMapT_.clear();
}


bool TePDIParameters::CheckParameter( const std::string& pname,
                                      TePDIParamTypes type ) const
{
  switch( type ) {
    case TePDIIntParam :
    {
      if( IntegerMap_.find( pname ) != IntegerMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIIntVectorParam :
    {
      if( IntegerVectorMap_.find( pname ) != IntegerVectorMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIRasterParam :
    {
      if( RasterMap_.find( pname ) != RasterMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIRasterVectorParam :
    {
      if( RasterVecMap_.find( pname ) != RasterVecMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIRastersVectorPtrParam :
    {
      if( RasterVecPtrMap_.find( pname ) != RasterVecPtrMap_.end() ){
        return true;
      }
      break;
    }    
    case TePDIStringParam :
    {
      if( StringMap_.find( pname ) != StringMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIDoubleParam :
    {
      if( DoubleMap_.find( pname ) != DoubleMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIDoubleVectorParam :
    {
      if( DoubleVectorMap_.find( pname ) != DoubleVectorMap_.end() ){
        return true;
      }
      break;
    }    
    case TeBoxParam :
    {
      if( PDIBoxMap_.find( pname ) != PDIBoxMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIRgbPaletteParam :
    {
      if( PDIRgbPaletteMap_.find( pname ) != PDIRgbPaletteMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIFilterMaskParam :
    {
      if( PDIFilterMaskMap_.find( pname ) != PDIFilterMaskMap_.end() ){
        return true;
      }
      break;
    }    
    case TePolygonParam :
    {
      if( TePolygonMap_.find( pname ) != TePolygonMap_.end() ){
        return true;
      }
      break;
    }    
    case TePDILutParam :
    {
      if( TePDILutMap_.find( pname ) != TePDILutMap_.end() ){
        return true;
      }
      break;
    }
    case TePDILutPtrParam :
    {
      if( TePDILutPtrMap_.find( pname ) != TePDILutPtrMap_.end() ){
        return true;
      }
      break;
    }    
    case TePDILayerParam :
    {
      if( TePDILayerMap_.find( pname ) != TePDILayerMap_.end() ){
        return true;
      }
      break;
    }    
    case TePolygonSetParam :
    {
      if( TePolygonSetMap_.find( pname ) != TePolygonSetMap_.end() ){
        return true;
      }
      break;
    }      
    case TePolygonSetPtrParam :
    {
      if( TePolygonSetPtrMap_.find( pname ) != TePolygonSetPtrMap_.end() ){
        return true;
      }
      break;
    }
    case TePDIHistogramPtrParam :
    {
      if( TePDIHistogramPtrMap_.find( pname ) != TePDIHistogramPtrMap_.end() ){
        return true;
      }
      break;
    }   
    case TeCoordPairVectPtrParam :
    {
      if( TeCoordPairVectPtrMap_.find( pname ) != 
        TeCoordPairVectPtrMap_.end() ){
        
        return true;
      }
      break;
    }
    case TePDIHistVecParam :
    {
      if( hist_vect_map_.find( pname ) != 
        hist_vect_map_.end() ){
        
        return true;
      }
      break;
    }
    case TePDIPolSetMapPtrParam :
    {
      if( TePDIPolSetMapPtrMapT_.find( pname ) != 
        TePDIPolSetMapPtrMapT_.end() ){
        
        return true;
      }
      break;
    }    
    default :
    {
      PDIAGN_LOG_AND_THROW( "Invalid Parameter Type" );
      break;
    }
  }

  return false;
}


void TePDIParameters::SetdecName( const std::string& dec_name )
{
  SetParameter( "dec_name", dec_name );
}


std::string TePDIParameters::decName() const
{
  std::string return_string;
  
  PDIAGN_TRUE_OR_THROW( GetParameter( "dec_name", return_string ),
    "Parameters descriptor not set" );
  
  return return_string;
}


bool TePDIParameters::operator==( const TePDIParameters& ext_instance ) const
{
  if( last_up_time_ == ext_instance.last_up_time_ ) {
    return true;
  } else {
    return false;
  }
}


bool TePDIParameters::operator!=( const TePDIParameters& ext_instance ) const
{
  if( last_up_time_ == ext_instance.last_up_time_ ) {
    return false;
  } else {
    return true;
  }
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TePDIRasterPtrType& data )
{
  RasterMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TePDIRasterPtrType& data ) const
{
  TePDIRasterMap::iterator it = RasterMap_.find( pname );

  if( it != RasterMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Raster reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TePDIRasterVectorType& data )
{
  RasterVecMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TePDIRasterVectorType& data ) const
{
  TePDIRasterVecMap::iterator it = RasterVecMap_.find( pname );

  if( it != RasterVecMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Raster vector parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TePDIRasterVectorPtrType& data )
{
  RasterVecPtrMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TePDIRasterVectorPtrType& data ) const
{
  TePDIRasterVecPtrMap::iterator it = RasterVecPtrMap_.find( pname );

  if( it != RasterVecPtrMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Rasters vector pointer parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  TePDIRgbPalette::const_pointer& data )
{
  PDIRgbPaletteMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDIRgbPalette::pointer& data ) const
{
  TePDIRgbPaletteMap::iterator it = PDIRgbPaletteMap_.find( pname );

  if( it != PDIRgbPaletteMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Palette reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    int data )
{
  IntegerMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    int& data ) const
{
  TePDIIntegerMap::iterator it = IntegerMap_.find( pname );

  if( it != IntegerMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Integer parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    std::vector< int >& data )
{
  IntegerVectorMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    std::vector< int >& data ) const
{
  TePDIIntegerVectorMap::iterator it = IntegerVectorMap_.find( pname );

  if( it != IntegerVectorMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Integer Vector parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const std::string& data )
{
  StringMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    std::string& data ) const
{
  TePDIStringMap::iterator it = StringMap_.find( pname );

  if( it != StringMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: String parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    double data )
{
  DoubleMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    double& data ) const
{
  TePDIDoubleMap::iterator it = DoubleMap_.find( pname );

  if( it != DoubleMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Double parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    std::vector< double >& data )
{
  DoubleVectorMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    std::vector< double >& data ) const
{
  TePDIDoubleVectorMap::iterator it = DoubleVectorMap_.find( pname );

  if( it != DoubleVectorMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: Double Vector parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TeBox& data )
{
  PDIBoxMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TeBox& data ) const
{
  TeBoxMap::iterator it = PDIBoxMap_.find( pname );

  if( it != PDIBoxMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TeBox parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePDIFilterMask::pointer& data )
{
  PDIFilterMaskMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePDIFilterMask::pointer& data ) const
{
  TePDIFilterMaskMap::iterator it = PDIFilterMaskMap_.find( pname );

  if( it != PDIFilterMaskMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( 
    "Invalid get: TePDIFilterMask reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePolygon& data )
{
  TePolygonMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePolygon& data ) const
{
  TePolygonMap::iterator it = TePolygonMap_.find( pname );

  if( it != TePolygonMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePolygon reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePDITypes::TePDILutType& data )
{
  TePDILutMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePDITypes::TePDILutType& data ) const
{
  TePDILutMap::iterator it = TePDILutMap_.find( pname );

  if( it != TePDILutMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePDILut reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePDITypes::TePDILutPtrType& data )
{
  TePDILutPtrMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePDITypes::TePDILutPtrType& data ) const
{
  TePDILutPtrMap::iterator it = TePDILutPtrMap_.find( pname );

  if( it != TePDILutPtrMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( 
    "Invalid get: TePDILut poiter reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePDITypes::TePDILayerPtrType& data )
{
  TePDILayerMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePDITypes::TePDILayerPtrType& data ) const
{
  TePDILayerMap::iterator it = TePDILayerMap_.find( pname );

  if( it != TePDILayerMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePDILut reference parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
                                    const TePolygonSet& data )
{
  TePolygonSetMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
                                    TePolygonSet& data ) const
{
  TePolygonSetMap::iterator it = TePolygonSetMap_.find( pname );

  if( it != TePolygonSetMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePolygonSet parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TePDIPolygonSetPtrType& data )
{
  TePolygonSetPtrMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TePDIPolygonSetPtrType& data ) const
{
  TePolygonSetPtrMap::iterator it = TePolygonSetPtrMap_.find( pname );

  if( it != TePolygonSetPtrMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePolygonSet pointer parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDIHistogram::pointer& data )
{
  TePDIHistogramPtrMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDIHistogram::pointer& data ) const
{
  TePDIHistogramPtrMap::iterator it = TePDIHistogramPtrMap_.find( pname );

  if( it != TePDIHistogramPtrMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePDIHistogram pointer parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TeCoordPairVectPtrType& data )
{
  TeCoordPairVectPtrMap_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TeCoordPairVectPtrType& data ) const
{
  TeCoordPairVectPtrMap::iterator it = TeCoordPairVectPtrMap_.find( pname );

  if( it != TeCoordPairVectPtrMap_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TeCoordPairVect pointer parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const std::vector< TePDIHistogram::pointer >& data )
{
  hist_vect_map_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  std::vector< TePDIHistogram::pointer >& data ) const
{
  HistVectMapT::iterator it = hist_vect_map_.find( pname );

  if( it != hist_vect_map_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePDIHistVecParam parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::SetParameter( const std::string& pname,
  const TePDITypes::TePDIPolSetMapPtrType& data )
{
  TePDIPolSetMapPtrMapT_[ pname ] = data;
  update_time();
}


bool TePDIParameters::GetParameter( const std::string& pname,
  TePDITypes::TePDIPolSetMapPtrType& data ) const
{
  TePDIPolSetMapPtrMapT::iterator it = TePDIPolSetMapPtrMapT_.find( pname );

  if( it != TePDIPolSetMapPtrMapT_.end() ) {
    data = it->second;
    return true;
  }

  PDIAGN_LOGWARN( "Invalid get: TePDIPolSetMapPtrParam parameter not stored" +
    std::string( " [" ) + pname + "]" );

  return false;
}


void TePDIParameters::update_time()
{
  last_up_time_ = time( 0 );
}
