/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/
#include <TeLegendEntry.h>

TeLegendEntry::TeLegendEntry (const TeLegendEntry& leg)
{
	//clears the current map
	TeGeomRepVisualMap::const_iterator currMapIt = visualMap_.begin();
	while( currMapIt != visualMap_.end() )
	{
		((TeVisual&)(currMapIt->second)).clear();
		++currMapIt;
	}
	visualMap_.clear();
		
	//copy the new map
	TeGeomRepVisualMap::const_iterator it = leg.visualMap_.begin();
	while( it != leg.visualMap_.end() )
	{
		visualMap_[ it->first ] = it->second;
		++it;
	}
	slice_ = leg.slice_;
	label_ = leg.label_;
	id_ = leg.id_;
	theme_ = leg.theme_;
	group_ = leg.group_;
}

TeLegendEntry&
TeLegendEntry::operator = (const TeLegendEntry& leg)
{
	if ( this != &leg )
	{	//clears the current map
		TeGeomRepVisualMap::const_iterator currMapIt = visualMap_.begin();
		while( currMapIt != visualMap_.end() )
		{
			((TeVisual&)(currMapIt->second)).clear();
			++currMapIt;
		}
		visualMap_.clear();
			
		//copy the new map
		TeGeomRepVisualMap::const_iterator it = leg.visualMap_.begin();
		while( it != leg.visualMap_.end() )
		{
			visualMap_[ it->first ] = it->second;
			++it;
		}
		slice_ = leg.slice_;
		label_ = leg.label_;
		id_ = leg.id_;
		theme_ = leg.theme_;
		group_ = leg.group_;
	}
	return *this;
}

void
TeLegendEntry::clear()
{
	slice_.from_.clear();
	slice_.to_.clear();
	label_.clear();

	//clears the current map
	TeGeomRepVisualMap::const_iterator currMapIt = visualMap_.begin();
	while( currMapIt != visualMap_.end() ){
		((TeVisual&)(currMapIt->second)).clear();
		++currMapIt;
	}
	visualMap_.clear();

}

void 
TeLegendEntry::color( TeColor& color )
{
	if(visualMap_.find(TePOLYGONS) != visualMap_.end())
	{
		TeVisual& tmpVisual = visualMap_[TePOLYGONS];
		tmpVisual.color( color );
		if ( tmpVisual.style() != 1 ) 
			tmpVisual.contourColor( color );
	}
	if(visualMap_.find(TeLINES) != visualMap_.end())
	{
		TeVisual& tmpVisual = visualMap_[TeLINES];
		tmpVisual.color( color );
	}
	if(visualMap_.find(TePOINTS) != visualMap_.end())	
	{
		TeVisual& tmpVisual = visualMap_[TePOINTS];
		tmpVisual.color( color );
	}
}
