/************************************************************************************
TerraLib - a library for developing GIS applications.
Copyright  2001-2004 INPE and Tecgraf/PUC-Rio.

This code is part of the TerraLib library.
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

You should have received a copy of the GNU Lesser General Public
License along with this library.

The authors reassure the license terms regarding the warranties.
They specifically disclaim any warranties, including, but not limited to,
the implied warranties of merchantability and fitness for a particular purpose.
The library provided hereunder is on an "as is" basis, and the authors have no
obligation to provide maintenance, support, updates, enhancements, or modifications.
In no event shall INPE and Tecgraf / PUC-Rio be held liable to any party for direct,
indirect, special, incidental, or consequential damages arising out of the use
of this library and its documentation.
*************************************************************************************/

#include "TeQuerierDBStr1.h"
#include "TeDatabase.h"
#include "TeTemporalSeries.h"
#include "TeSTInstance.h"

bool 
TeQuerierDBStr1::initPortal(TeRepresentation& rep, TeTSEntry* ent)
{
	string selectClause, fromClause, whereClause, orderByClause, groupByClause, parClause;
	isGroup_ = false;

	//------------------------------------------- begin get information
	//get database
	TeDatabase* db = params_.theme()->layer()->database();

	//get attribute tables
	TeAttrTableVector atts; 
	params_.theme()->getAttTables(atts); 
	if(atts.empty())
		return false;

	//verify if will be group the objects	
	TeGroupingAttr groups = params_.groupAttr();  
	bool groupAttr = !(groups.empty());
		
	//verify if the theme has collection table
	bool hasCollTable = false;
	if(db->tableExist(params_.theme()->collectionAuxTable()))
		hasCollTable = true;
	
	clearVectors();
	params_.setLoadAttrs(params_.loadSetedAttrs()); //the original data
	
	//get the link name - the first attribute table
	linkName_ = atts[0].name() +"."+ atts[0].linkName();
	
	if(params_.loadGeom())
		geomRepr_.push_back(rep);
	
	//------------------------------------------- end get information
	
	//------------------------------------------- begin information to mount SQL
	// -------- from
	bool fromNeedGeomTable = false;
	bool fromNeedCollTable = false;
	
	if(params_.loadGeom() || params_.hasSpatialRes())
		fromNeedGeomTable = true;
	if(hasCollTable)
		fromNeedCollTable = true;

	// -------- group and order by 
	bool useGroupByClause = false;	//group by object_id
	bool useOrderByClause = false;  //order by object_id, geom_id
	//Use group by clause when 
	//1)it must group the attributes and 
	//2)it must not load geometry and 
	//3)all statistic functions exist in the SGBD 
	string sGroup="";
	if(groupAttr)
	{
		if(params_.loadGeom())
			useGroupByClause = false; 
		else
		{
			sGroup = db->getSQLStatistics(groups);
			isGroup_ = true;
			TeGroupingAttr::iterator it = groups.begin();
			while(it!= groups.end())
			{
				if(it->second != TeNOSTATISTIC)
					isGroup_ = false;
				++it;
			}
			if(isGroup_)
				useGroupByClause = true; 
		}
		groupInMemory_ = !useGroupByClause;
	}
	else
		useOrderByClause = true;

	
	// -------- select
	bool selectNeedGeom = false;
	if(params_.loadGeom())
		selectNeedGeom = true;
	
	//------------------------------------------- end information to mount SQL
	
	//------------------------------------------- begin mount SQL
	// order and group by clause
	orderByClause = " ORDER BY "+ linkName_; 
	if(selectNeedGeom)
		orderByClause += ", "+ rep.tableName_ +".geom_id";
	groupByClause = " GROUP BY "+ linkName_;

	// ------------- select clause
	bool hasLinkName = false;
	if(groupAttr)
	{
		if(!groupInMemory_) //use statistic function database
		{
			selectClause  = sGroup;
			selectClause += ", "+ linkName_;  
			hasLinkName = true;
		}
		else //load the attributes from TeGroupingAttr
		{
			//fill select clause from set of attributes
			string lastAttr = "";
			TeGroupingAttr::iterator it = groups.begin(); 
			while(it!= groups.end())
			{
				if(lastAttr != it->first.name_)
				{
					if(it != groups.begin())
						selectClause += ",";

					selectClause += it->first.name_;
				}
				lastAttr = it->first.name_;
				++it;
			}
			
			// select datatime information
			if(!attrTable_.name().empty())
			{
				selectClause += ", "+ attrTable_.name()+"."+attrTable_.attInitialTime();
				if(attrTable_.attInitialTime()!=attrTable_.attFinalTime())
					selectClause += ", "+ attrTable_.name()+"."+attrTable_.attFinalTime();
			}

			if(!selectClause.empty())
				selectClause += ", ";
			selectClause += linkName_;
			hasLinkName = true;
		}
	}
	else // load all attributes or the attributes that are in the vector
	{
		if(atts.empty() && (params_.loadAllAttr() || (!params_.loadAttrs().empty())))
			return false;
			
		// get some information about the attribute tables required
		for(unsigned int i=0; i<atts.size(); i++)
		{
			string initialTime, finalTime;

			if((atts[i].tableType()==TeAttrEvent) || (atts[i].tableType()==TeFixedGeomDynAttr))
			{
				attrTable_ = atts[i]; 
				initialTime = attrTable_.attInitialTime();
				finalTime = attrTable_.attFinalTime();
			}
		
			// fill vector of unique name 
			string unName = TeConvertToUpperCase(atts[i].uniqueName());
			string unNameT = TeConvertToUpperCase(atts[i].name()+"."+atts[i].uniqueName());
			uniqueNames_.push_back(unName);
			uniqueNames_.push_back(unNameT);

			//fill vector of attributes and sql string with all attributes
			if(params_.loadAllAttr())
			{
				TeAttributeList::iterator itAttr = atts[i].attributeList().begin();
				while(itAttr!= atts[i].attributeList().end())
				{
					string attribute = atts[i].name() +"."+ (*itAttr).rep_.name_;
					params_.loadAttrs().push_back(attribute);
					params_.loadAttrs().push_back((*itAttr).rep_.name_);
				
					//select clause
					if(!selectClause.empty())
						selectClause += ", ";
					selectClause += attribute; 
					if(attribute==linkName_)
						hasLinkName = true;
					++itAttr;
				}
			}
			//only fill the sql string with the attributes of the TeSTInstance
			else // make sure the unique and temporal columns are retrieved
			{
				string un = atts[i].name()+"."+ atts[i].uniqueName();
				if (find(params_.loadAttrs().begin(), params_.loadAttrs().end(), un) == params_.loadAttrs().end())
				{
					if(!selectClause.empty())
						selectClause += ", ";
					selectClause += un;
					if(un==linkName_)
						hasLinkName = true;
				}

				string ini = atts[i].name()+"."+initialTime;
				if (!initialTime.empty() && (find(params_.loadAttrs().begin(), params_.loadAttrs().end(), ini) == params_.loadAttrs().end()))
				{
					if(!selectClause.empty())
						selectClause += ", ";
					selectClause += ini;
				}

				string fin = atts[i].name()+"."+finalTime;
				if	(!finalTime.empty() &&  initialTime != finalTime && 
					(find(params_.loadAttrs().begin(), params_.loadAttrs().end(), fin) == params_.loadAttrs().end()))
				{
					if(!selectClause.empty())
						selectClause += ", ";
					selectClause += fin;
				}
			}
		}

		if(!params_.loadAllAttr())
		{
			vector<string>  newNames;
			vector<string>::iterator itVec = params_.loadAttrs().begin();
			while(itVec!=params_.loadAttrs().end())
			{
				//insert in select clause
				if(!selectClause.empty())
						selectClause += ", ";
				selectClause += (*itVec);
				if((*itVec)==linkName_)
					hasLinkName = true;
			
				//insert in the attribute name vector without table name
//				size_t pos = (*itVec).find(".", string::npos,1);
				size_t pos = (*itVec).find(".", 0, 1);
				if (pos != string::npos)
				{
					string newNam = (*itVec).substr(pos+1);
					newNames.push_back(newNam);
				}
				++itVec;
			}

			itVec = newNames.begin();
			while(itVec!=newNames.end())
			{
				params_.loadAttrs().push_back(*itVec);
				++itVec;
			}

			if(!hasLinkName)
			{
				if(!selectClause.empty())
					selectClause += ", ";
				selectClause += linkName_;
			}
		}
	}
	
	if(selectNeedGeom)
	{
		if(!selectClause.empty())
			selectClause += ", ";

		TeAttributeList attrs;
		if (!db->getAttributeList(rep.tableName_, attrs))
			return false; 

		for(unsigned int i=0; i<attrs.size(); ++i)
		{
			if(i>0)
				selectClause += ",";
			selectClause += rep.tableName_+"."+ attrs[i].rep_.name_;
		}
		// order by clause
		if((rep.geomRep_ == TePOLYGONS) && (db->dbmsName() != "OracleSpatial") && (db->dbmsName() != "PostGIS") )
			orderByClause += ", parent_id ASC, num_holes DESC";
	}
	
	// ------------- from and where clause
	if(!fromNeedCollTable)
	{
		if(fromNeedGeomTable)
			fromClause = " FROM " + tableJoin(atts, rep.tableName_, "object_id");
		else
			fromClause = " FROM " + tableJoin(atts);
		
		//if the theme does not have collection, it must apply its restrictions (attribute, temporal and spatial)
		whereClause = params_.theme()->sqlWhereRestrictions(&rep); 
	}
	else
	{
		if(fromNeedGeomTable)
			fromClause = params_.theme()->sqlGridFrom(rep.tableName_);
		else
			fromClause = params_.theme()->sqlGridFrom();
	}

	if(ent)
	{
		if(attrTable_.name().empty())
			params_.theme()->getTemporalTable(attrTable_);

		string iniTime = attrTable_.name()+"."+attrTable_.attInitialTime(); 
		string finTime = attrTable_.name()+"."+attrTable_.attFinalTime(); 
		if(!whereClause.empty())
			whereClause += " AND ";

		if ((params_.chronon()==TeMONTHOFYEAR) || (params_.chronon()==TeDAYOFWEEK))
			whereClause += db->getSQLTemporalWhere(ent->timeInt_, ent->timeInt_, params_.chronon(), TeTIMEDURING, iniTime, finTime);    
		else
		{
			TeTimeInterval interval = ent->time_;
			interval.intervalChronon(params_.chronon());

			whereClause += db->getSQLTemporalWhere(interval, TeTIMEDURING, iniTime, finTime);
		}
	}

	if(!objectId().empty())
	{
		if(!whereClause.empty())
			whereClause += " AND ";
		
		whereClause += linkName_ +" = '"+ objectId() +"'";  
	}

	//---------- querier restriction
	string sqlQuerierRest = sqlWhereRestrictions(&rep);
	if(!whereClause.empty())
		whereClause += " AND ";

	whereClause += sqlQuerierRest;

	fromClause_ = fromClause;
	whereClause_ = whereClause;
    string sql = "SELECT "+ selectClause + fromClause;
	if(!whereClause.empty())
		sql += " WHERE "+ whereClause;
	
	if(useGroupByClause)
		sql += " "+ groupByClause;
	else
		sql += " "+ orderByClause;
	
	//------------------------------------------- end mount SQL

	// --------- Submit the query
	portals_.clear();
	TeDatabasePortal* portal = db->getPortal();
	
	if(!portal)
		return false;
	
	if(!portal->query(sql))
	{
		delete (portal);
		flagPortal_ = false;
		return false;
	}

	if(!portal->fetchRow())
	{
		delete (portal);
		flagPortal_ = false;
		return false;
	}

	string id = portal->getData(linkName_);
	bool flag = true;
	while(id.empty() && flag) 
	{
		flag = portal->fetchRow();
		if(flag)
			id = portal->getData(0); //object_Id
	}
	
	if(id.empty() || (!flag))
	{
		delete (portal);
		flagPortal_ = false;
		return false;
	}

	portals_.push_back (portal);
	flagPortal_ = true;
	lastObjId_ ="";
	return true;
}


bool 
TeQuerierDBStr1::initGeomPortal(TeRepresentation& rep, TeTSEntry* ent)
{
	string selectClause, fromClause, whereClause, orderByClause, parClause;
	string initialTime, finalTime;
	string uniqueName;
	
	TeDatabase* db = params_.theme()->layer()->database();

	//------- Get geometry table
	geomRepr_.push_back(rep);
	if(rep.tableName_.empty())
		return false;
	
	selectClause = linkName_ +","+ rep.tableName_ +".* ";
	orderByClause = " ORDER BY "+ linkName_;  

	if((rep.geomRep_ == TePOLYGONS) && (db->dbmsName() != "OracleSpatial") && (db->dbmsName() != "PostGIS") )
		orderByClause += " , parent_id ASC, num_holes DESC";
	
	//------- Get temporal attribute 
	if(!attrTable_.name().empty())
	{
		initialTime = attrTable_.name() +"."+ attrTable_.attInitialTime ();
		finalTime = attrTable_.name() +"."+ attrTable_.attFinalTime ();
			
		// fill vector of unique name 
		uniqueName = TeConvertToUpperCase(attrTable_.name()+"."+attrTable_.uniqueName());
						
		selectClause += ", "+ attrTable_.name()+"."+ attrTable_.uniqueName();
		selectClause += ", "+ attrTable_.name()+"."+initialTime;

		if (initialTime != finalTime)
			selectClause += ", "+ attrTable_.name()+"."+finalTime;
	}
	
	if(!db->tableExist(params_.theme()->collectionAuxTable()))
	{
		TeAttrTableVector atts; 
		params_.theme()->getAttTables(atts); 

		if(atts.empty())
			return false;

		fromClause = " FROM " + tableJoin(atts, rep.tableName_, "object_id");
		whereClause = params_.theme()->sqlWhereRestrictions(&rep);
	}
	else
		fromClause = params_.theme()->sqlGridFrom(rep.tableName_);
	
	if(ent)
	{
		string iniTime = attrTable_.name()+"."+initialTime;
		string finTime = attrTable_.name()+"."+finalTime;
		if(!whereClause.empty())
			whereClause += " AND ";

		if ((params_.chronon()==TeMONTHOFYEAR) || (params_.chronon()==TeDAYOFWEEK))
			whereClause += db->getSQLTemporalWhere(ent->timeInt_, ent->timeInt_, params_.chronon(), TeTIMEDURING, iniTime, finTime);    
		else
		{
			TeTimeInterval interval = ent->time_;
			interval.intervalChronon(params_.chronon());

			whereClause += db->getSQLTemporalWhere(interval, TeTIMEDURING, iniTime, finTime);
		}
	}

	//where clause
	if(!objectId().empty())
	{
		if(!whereClause.empty())
			whereClause += " AND ";
		
		whereClause += rep.tableName_+".object_id = '"+ objectId() +"'";  
	}

	//---------- querier restriction
	string sqlQuerierRest = sqlWhereRestrictions(&rep);
	if(!whereClause.empty())
		whereClause += " AND ";

	whereClause += sqlQuerierRest;

	//----------

	string sql = "SELECT "+ selectClause + fromClause;
	if(!whereClause.empty())
		sql += " WHERE "+ whereClause;
	sql += orderByClause;

	//------ Submit the query
	TeDatabasePortal* portal = db->getPortal();
	if(!portal)
		return false;
	
	if(!portal->query(sql))
	{
		delete portal;
		return false;
	}

	if(!portal->fetchRow())
	{
		delete portal;
		return false;
	}

	portals_.push_back (portal);
	return true;
}


// ---------- final - initPortal

// It is used in two cases:
// 1) when it must return all instances of an object 
// 2) when the attributes was grouped by sql functions - without geometry 

bool
TeQuerierDBStr1::fillSTOGrouped(TeSTInstance& sto, bool fetchInstance)
{
	if(portals_.empty()) 
		return false;

	TeDatabasePortal* portal = portals_[0]; 
	if(!portal) 
		return false; 

	TeRepresentation* rep = 0;
	if(!geomRepr_.empty())
		rep = &(geomRepr_[0]);

	if(!flagPortal_)
	{
		clearVectors();
		return false;
	}

	TeTime  minT1, maxT2;
	
	// builds an ST object instance
	TeTime t1, t2;
	TePropertyVector prop;

	// verify if exist several geometries for an object
	bool flagGeom = true;
	while(flagGeom && fetchInstance)
	{
		sto.objectId (portal->getData(linkName_));
						
		if(rep && (!params_.loadGeom()))
		{
			int geomId = portal->getInt(rep->tableName_ +".geom_id");
			
			if(sto.objectId()==lastObjId_) 
			{
				if (geomId_!=geomId)
				{
					flagGeom = portal->fetchRow();
					continue;
				}
			}
			else
				geomId_ = geomId;
		}
		flagGeom = false;
	}
	
	
	// ------------- begin fill attribute
	// process the records filling the parameters of each instance
	TeAttributeList& attrsPortal = portal->getAttributeList();
	TeAttributeList::iterator it= attrsPortal.begin();

	if(isGroup_)  // it was grouped by sql functions and without geometry 
	{
		sto.addUniqueId(string(portal->getData(linkName_)));

		unsigned int size = params_.groupAttr().size();
		for(unsigned int s=0; s<size; ++s)
		{
			TeProperty p;
			p.attr_ = attrsPortal[s];   
			p.value_ = portal->getData (s);
			prop.push_back(p);
		}
	}
	else
	{
		while(it!=attrsPortal.end())
		{
			TeProperty p;
			string name = (*it).rep_.name_;
			p.attr_ = (*it);

			if(name==attrTable_.attInitialTime())
			{
				t1 = portal->getDate(attrTable_.attInitialTime());
				if(attrTable_.attInitialTime()==attrTable_.attFinalTime())
				{
					t2 = t1;
					sto.timeInterval(TeTimeInterval(t1,t2));
				}
			}
			else if(name==attrTable_.attFinalTime())
			{
				t2 = portal->getDate(attrTable_.attFinalTime());
				sto.timeInterval(TeTimeInterval(t1,t2));
			}
			else if(find(uniqueNames_.begin(), uniqueNames_.end(), TeConvertToUpperCase(name)) != uniqueNames_.end())
			{
				string uniqueValue = portal->getData(name);
				sto.addUniqueId(uniqueValue);
			}
			
			if(find(params_.loadAttrs().begin(), params_.loadAttrs().end(), name) != params_.loadAttrs().end())
			{
				p.value_ = portal->getData(name);
				prop.push_back(p);
			}
			++it;
		}
	}

	//! Set the property in the stoInstance
	sto.properties(prop);	
	sto.theme(params_.theme());

	if(!fetchInstance) //only to get the attributes
		return true;

	// ------------- end fill attribute
	//total time
	if(t1.isValid() && t1<minT1)
		minT1 = t1;
	if(t2.isValid() && maxT2<t2)
		maxT2 = t2;
		
	// ------------- begin fill geometries
	if(params_.loadGeom())
	{
		if(t1.isValid() && t2.isValid())
			flagPortal_ = addGeometry(portal, rep->geomRep_, sto, linkName_, /*rep->tableName_,*/ TeTimeInterval(t1,t2), attrTable_.attInitialTime(), attrTable_.attFinalTime());
		else
			flagPortal_ = addGeometry(portal, rep->geomRep_, sto, linkName_ /*rep->tableName_*/);
	}
	else
		flagPortal_ = portal->fetchRow ();

	// ------------- end fill geometries
	
	lastObjId_ = sto.objectId(); 
	return true;
}

bool 
TeQuerierDBStr1::fillGeomSTO(TeSTInstance& sto, unsigned int index)
{
	if((portals_.size()<(index+1)) || (geomRepr_.size()<(index+1)))
		return false;

	TeDatabasePortal* portal = portals_[index]; 
	TeRepresentation rep = geomRepr_[index];
	if(!portal)
		return false; 

	TeTimeInterval time = sto.timeInterval();
	bool flag = false;
	if(time.isValid())
		flag = addGeometry(portal, rep.geomRep_, sto, linkName_, /*rep->tableName_,*/ time, attrTable_.attInitialTime(), attrTable_.attFinalTime());
	else
		flag = addGeometry(portal, rep.geomRep_, sto, linkName_ /*rep->tableName_*/);

	if(flag==false)
		flagPortal_ = false; 
	
	return true;
}

// It is used in a case:
// 1) when the attributes of the object instances must be grouping in memory - with or without geometry 
bool
TeQuerierDBStr1::fillSTONoGrouped(TeSTInstance& sto)
{
	int s;
	if(portals_.empty())
		return false;

	TeDatabasePortal* portal = portals_[0]; 
	if(!portal) 
		return false; 

	TeRepresentation* rep = 0; 
	if(!geomRepr_.empty())
		rep = &(geomRepr_[0]);
	
	//verify if the portal points to any STO
	if(!flagPortal_)
	{
		clearVectors();
		return false;
	}

	// how many attributes must be grouped from portal
	string lastAttr = "";
	int numberAttrs = 0;
	TeGroupingAttr::iterator it = (params_.groupAttr()).begin(); 
	while(it!= (params_.groupAttr()).end())
	{
		if(lastAttr != it->first.name_)
			++numberAttrs;
		lastAttr = it->first.name_;
		++it;
	}

	TePropertyVector prop;
	map<int, vector<double> >	valuesDouble;
	map<int, vector<string> >	valuesString;
	string lastObj = "";

	TeAttributeList& attrsPortal = portal->getAttributeList();
	
	if(!params_.loadGeom())
	{
		int cont = 0;
		do
		{
			string objId = portal->getData(linkName_);
			if((!lastObj.empty()) && (objId != lastObj))
				break;  
					
			string val = ""; 
			//portal has the attributes that will be grouped 
			for(s=0; s<numberAttrs; ++s)  
			{
				val = portal->getData(s);
				if((attrsPortal[s].rep_.type_==TeREAL) || (attrsPortal[s].rep_.type_==TeINT))
				{
					if(val.empty())
						valuesDouble[s].push_back (TeMAXFLOAT);  //invalid value
					else
						valuesDouble[s].push_back (atof(val.c_str()));
				}
				else
					valuesString[s].push_back(val);
			}

			flagPortal_ = portal->fetchRow();
			lastObj = objId;
			++cont;

		} while(flagPortal_);
	}
	else // -- when it must fill the geometry
	{
		
		//------- first: load the geometry, attributes and time of the first object 
		//object id
		lastObj = portal->getData(linkName_);
		
		//attribute val
		string val = ""; 
		//unsigned int size = attrsPortal.size();
		for(s=0; s<numberAttrs/*(size-1)*/; ++s)
		{
			val = portal->getData(s);
			if((attrsPortal[s].rep_.type_==TeREAL) || (attrsPortal[s].rep_.type_==TeINT))
			{
				if(val.empty())
					valuesDouble[s].push_back (TeMAXFLOAT);  //invalid value
				else
					valuesDouble[s].push_back (atof(val.c_str()));
			}
			else
				valuesString[s].push_back (val);
		}

		//time
		TeTime t1, t2;
		if(!attrTable_.name().empty())
		{
			t1 = portal->getDate(attrTable_.attInitialTime());
			t2 = portal->getDate(attrTable_.attFinalTime());
		}
		
		sto.objectId(lastObj);
		if(t1.isValid() && t2.isValid())
			flagPortal_ = addGeometry(portal, rep->geomRep_, sto, linkName_, /*rep->tableName_,*/ TeTimeInterval(t1,t2), attrTable_.attInitialTime(), attrTable_.attFinalTime());
		else
			flagPortal_ = addGeometry(portal, rep->geomRep_, sto, linkName_ /*rep->tableName_*/);
		
		//------- second: load the attributes of the other objects
		int cont = 0;
		while (flagPortal_)
		{
			string objId = portal->getData(linkName_);
			if(objId != lastObj)
				break;  
			
			//same object id
			string gId = portal->getData (rep->tableName_ +".geom_id");
			int geomId = atoi(gId.c_str());
			if(gId.empty())
				geomId = cont;   
			
			if(!cont)
				geomId_ = geomId;  //first geom_id 
			
			//verify the geomId
			if(geomId!=geomId_)
			{
				flagPortal_ = portal->fetchRow();
				lastObj = objId;
				continue;
			}
				
			val = ""; 
			for(s=0; s<numberAttrs/*(size-1)*/; ++s)
			{
				val = portal->getData(s);
				if((attrsPortal[s].rep_.type_==TeREAL) || (attrsPortal[s].rep_.type_==TeINT))
				{
					if(val.empty())
						valuesDouble[s].push_back (TeMAXFLOAT);  //invalid value
					else
						valuesDouble[s].push_back (atof(val.c_str()));
				}
				else
					valuesString[s].push_back (val);
			}

			flagPortal_ = portal->fetchRow();
			lastObj = objId; 
			++cont;
		}
	}

	//fill the property from portal 
	//calculate the statistics
	lastAttr = "";
	unsigned int j=0;
	string colName = "";
	it = params().groupAttr().begin(); 

	TeStatisticValMap		statDouble;
	TeStatisticStringValMap statString;
	
	TeAttrDataType attrType = TeSTRING;
	while(it!=params().groupAttr().end())
	{
		if(lastAttr != it->first.name_)
		{
			statDouble.clear();
			statString.clear();
			//if((it->first.type_==TeREAL) || (it->first.type_==TeINT))
			if((attrsPortal[j].rep_.type_==TeREAL) || (attrsPortal[j].rep_.type_==TeINT))
			{
				TeCalculateStatistics(valuesDouble[j].begin(), valuesDouble[j].end(), statDouble);
				attrType = TeREAL;
			}
			else
				TeCalculateStatistics(valuesString[j].begin(), valuesString[j].end(), statString);
			
			colName = attrsPortal[j].rep_.name_;
			++j;
		}

		double resultD=0;
		string resultS="";
		string prefix;
		switch(it->second)
		{
			case TeCOUNT:
				resultD = statDouble[TeCOUNT]; 
				resultS = statString[TeCOUNT];
				prefix = "count";
				break;
			case TeVALIDCOUNT:
				resultD = statDouble[TeVALIDCOUNT]; 
				resultS = statString[TeVALIDCOUNT];
				prefix = "valCount";
				break;
			case TeMINVALUE:
				resultD = statDouble[TeMINVALUE];
				resultS = statString[TeMINVALUE];
				prefix = "min";
				break;
			case TeMAXVALUE:
				resultD = statDouble[TeMAXVALUE];
				resultS = statString[TeMAXVALUE];
				prefix = "max";
				break;
			case TeSUM:
				resultD = statDouble[TeSUM]; 
				prefix = "sum";
				break;
			case TeMEAN:
				resultD = statDouble[TeMEAN]; 
				prefix = "mean";
				break;
			case TeSTANDARDDEVIATION:
				resultD = statDouble[TeSTANDARDDEVIATION];
				prefix = "stDeviation";
				break;
			case TeVARIANCE:
				resultD = statDouble[TeVARIANCE];
				prefix = "variance";
				break;
			case TeSKEWNESS:
				resultD = statDouble[TeSKEWNESS];
				prefix = "skwness";
				break;
			case TeKURTOSIS:
				resultD = statDouble[TeKURTOSIS]; 
				prefix = "kurtosis";
				break;
			case TeAMPLITUDE:
				resultD = statDouble[TeAMPLITUDE]; 
				prefix = "amplitude";
				break;
			case TeMEDIAN:
				resultD = statDouble[TeMEDIAN]; 
				prefix = "median";
				break;
			case TeVARCOEFF:
				resultD = statDouble[TeVARCOEFF]; 
				prefix = "varcoeff";
				break;
			case TeMODE: 
				resultD = statDouble[TeMODE]; 
				prefix = "mode";
				break;
			default:
				break;
		}
		
		lastAttr = it->first.name_; 
		++it;

		//fill the property 
		TeProperty p;
		p.attr_.rep_.name_ = prefix +"_"+ colName;
		if(attrType == TeREAL)
		{
			p.attr_.rep_.type_ = TeREAL;
			p.value_ = Te2String(resultD);
		}
		else
		{
			p.attr_.rep_.type_ = TeSTRING;
			p.value_ = resultS;
		}
		
		prop.push_back(p);
	}
	
	//adds the property in the stoInstance
	sto.properties(prop);	
	sto.theme(params_.theme());
	sto.addUniqueId(lastObj);
	sto.objectId(lastObj);

	lastObjId_ = sto.objectId(); 
	return true;
}


bool
TeQuerierDBStr1::loadInstances(TeTSEntry* ent)
{
	if(!params_.theme())
		return false;

	TeRepresPointerVector repres = params_.theme()->layer()->vectRepres();
	TeRepresentation rep;
	
	//fill instances - first fill the geometry representation of the spatial restriction 
	if(params_.theme()->hasSpatialRest())
		rep = **(TeFindRepresentation(repres.begin(), repres.end(), params_.theme()->geomRepRestriction()));
	else
	{
		for(int i=0; i<(int)repres.size(); ++i)
		{
			if(repres[i]->geomRep_==TeTEXT)
				continue; 
			
			rep = *repres[i];
			break;
		}
	}
	
	if(!initPortal(rep, ent))
		return false;

	//get the attribute list
	attrList_.clear();
	if(!groupInMemory_)
	{
		TeSTInstance i;
		fillSTOGrouped(i, false);
		TePropertyVector props = i.getPropertyVector();
		for(int c=0; c<(int)props.size(); ++c)
			attrList_.push_back (props[c].attr_);
	}
	else
	{
		TeGroupingAttr::iterator it = params().groupAttr().begin();
		while(it!=params().groupAttr().end())
		{
			TeAttribute at;
			at.rep_ = it->first;
			attrList_.push_back (at);
			++it;
		}
	}
			
	if(!params_.loadGeom())
		return true;
	
	for(unsigned int i=0; i<repres.size(); ++i)
	{
		if( (repres[i]->geomRep_ == rep.geomRep_) || 
			(repres[i]->geomRep_ == TeTEXT) )
			continue;

		if(!initGeomPortal(*(repres[i]), ent))
		{
			clearVectors();
			return false;
		}
	}

	return true;
}

bool 
TeQuerierDBStr1::fetchInstance(TeSTInstance&  sto)  
{
	sto.clear();
	bool flag = false;
	
	if(groupInMemory_)
		flag = fillSTONoGrouped(sto); 
	else
		flag = fillSTOGrouped(sto); 
	
	if((flag==false) || (!params_.loadGeom()))
		return flag;
	
	for(unsigned int i=1; i<portals_.size(); ++i)
		flag = fillGeomSTO(sto, i);
	
	return flag;
}


int 
TeQuerierDBStr1::numElemInstances()
{
	//from clause has the geom table
	if(params_.loadGeom() || params_.hasSpatialRes())
	{
		//load the attribute tables
		TeAttrTableVector atts; 
		params_.theme()->getAttTables(atts); 
		if(atts.empty())
			return 0;

		string selectClause = " DISTINCT ";
		for(unsigned int i=0; i<atts.size(); ++i)
		{
			if(i>0)
				selectClause+= ",";
			selectClause+= atts[i].name() +"."+ atts[i].linkName();

			if(atts[i].linkName()!=atts[i].uniqueName())
				selectClause+= ","+ atts[i].name() +"."+ atts[i].uniqueName();
		}
		
        string sql = "SELECT "+ selectClause + fromClause_;
		if(!whereClause_.empty())
			sql += " WHERE "+ whereClause_;

		TeDatabasePortal* portal = params_.theme()->layer()->database()->getPortal();
		if(!portal)
			return 0;
		
		if(!portal->query(sql) || !portal->fetchRow())
		{
			delete (portal);
			return 0;
		}
		int result = portal->numRows();
		delete portal;
		return result;
	}
		
	if((portals_.empty()))
		return 0;

	//get the first portal
	if(!portals_[0])
		return 0;

	return (portals_[0]->numRows());
}




