
#include "TeSTEvent.h"
#include "TeSTInstance.h"
#include "TeTheme.h"
#include "TeTimeInterval.h"
	

TeSTEvent::TeSTEvent(const string& objId, const TePoint& point, const long& time):
		object_id_(objId),
		point_(point),
		time_(time)
{
	point_.objectId (objId);
	unique_id_.push_back (objId);
}

TeSTEvent::TeSTEvent(const string& objId, const TePoint& point, const vector<double>& attrValue, 
					 const vector<TeAttributeRep>& attrRep, const long& time):
		object_id_(objId),
		point_(point),
		time_(time),
		attrValue_(attrValue),
		attrRep_(attrRep)
{
	point_.objectId (objId);
	unique_id_.push_back (objId);
}

TeSTEvent::TeSTEvent(const TeSTEvent& other) 
{	
	unique_id_		= other.unique_id_;
	object_id_		= other.object_id_;
	time_			= other.time_;
	point_			= other.point_;  
	attrValue_		= other.attrValue_;
	attrRep_		= other.attrRep_;
	theme_			= other.theme_;
	attrOut_		= other.attrOut_;
}
			
TeSTEvent&					
TeSTEvent::operator= (const TeSTEvent& other)
{	
	if ( this != &other )
	{
		unique_id_		= other.unique_id_;
		object_id_		= other.object_id_;
		time_			= other.time_;
		point_			= other.point_;  
		attrValue_		= other.attrValue_;
		attrRep_		= other.attrRep_;
		theme_			= other.theme_;
		attrOut_		= other.attrOut_;
	}
	return *this;
}
	
	
bool 
TeSTEvent::operator== (const TeSTEvent& other) const
{	
	if (this->object_id_ != other.object_id_ || 
		!(this->time_ == other.time_) ||
		(this->point_.location().x() != other.point_.location().x()) ||
		(this->point_.location().y() != other.point_.location().y()))
		return false;

	return true; 
}


TeTheme* 
TeSTEvent::theme()
{	
	return theme_; 
}

void 
TeSTEvent::theme(TeTheme* t)
{
	theme_ = t;
}

TeTimeInterval 
TeSTEvent::timeInterval() 
{
	TeTimeInterval t;
	return t;
}

int 
TeSTEvent::getPropertyIndex(const string& name)
{
	vector<TeAttributeRep>::iterator itRep = attrRep_.begin();
	int index=-1;
	while (itRep!=attrRep_.end())
	{
		++index;
		if(itRep->name_==name)
			return index;
	}
	return index;
}


void 
TeSTEvent::setPropertyValue(const int& indexAttr, const double& value)
{
	if(attrValue_.empty())
		return; 

	if(indexAttr >(int)(attrValue_.size()-1))
		return;

	attrValue_[indexAttr] = value;
}

void 
TeSTEvent::addProperty(const TeProperty& prop, bool attrIn) 
{
	attrValue_.push_back(atof(prop.value_.c_str()));
	attrRep_.push_back(prop.attr_.rep_);
	if(!attrIn)
		attrOut_.push_back((attrValue_.size()-1));
}

void 
TeSTEvent::addProperty(const double& val, TeAttributeRep rep, bool attrIn)
{
	attrValue_.push_back(val);
	attrRep_.push_back(rep);
	if(!attrIn)
		attrOut_.push_back((attrValue_.size()-1));
}

TePropertyVector 
TeSTEvent::getOutPropertyVector()
{
	TePropertyVector vec;
	for(unsigned int i=0; i<attrOut_.size(); ++i)
	{
		TeProperty prop;
		prop.attr_.rep_ = attrRep_[attrOut_[i]];
		prop.value_ = Te2String(attrValue_[attrOut_[i]]);
		vec.push_back (prop);
	}
	return vec;
}

TePropertyVector 
TeSTEvent::getPropertyVector()
{
	TePropertyVector vec;
	for(unsigned int i=0; i<attrValue_.size(); ++i)
	{
		TeProperty prop;
		prop.attr_.rep_ = attrRep_[i];
		prop.value_ = Te2String(attrValue_[i]);
		vec.push_back (prop);
	}
	return vec;
}

bool 
TeSTEvent::getPropertyValue (string& val, int i)
{
	if ((i>=0) && (i<(int)attrValue_.size()))
	{
		val = Te2String(attrValue_[i]);
		return true;
	}
	else
		return false;
}

//------------------------------------------ TeSTEventImpl
bool 
TeSTEventSetImpl::search(const TeBox& b, vector<TePoint>& result)
{
	if(!kdTreeBuilt_ || !eventTree_)
		return false;

	vector<kdNode*> nodes;
	eventTree_->search(b, nodes);
	for(unsigned int i=0; i<nodes.size(); ++i)
	{
		for(unsigned int j=0; j<nodes[i]->getData().size(); ++j)
			result.push_back((nodes[i]->getData())[j]); 
	}
	return true;
}

bool 
TeSTEventSetImpl::nearestNeighbourSearch(const TeCoord2D& coord, vector<TePoint>& result, vector<double>& distances, const unsigned int& k)
{
	if(!kdTreeBuilt_ || !eventTree_)
		return false;

	eventTree_->nearestNeighborSearch(coord, result, distances, k); 
	return true;
}


//------------------------------------------ TeSTEventVector

TeSTEventVector::TeSTEventVector(): TeSTEventSetImpl()
{ }


TeSTEventVector::TeSTEventVector(const TeBox& box): TeSTEventSetImpl(box)
{ }

bool 
TeSTEventVector::getAttributeValue (const string& object_id, const int& attr_index, double& val, const long& /*time*/)
{
	STElemVecIterator it = events_.begin();
	while(it!=events_.end())
	{
		if(TeConvertToUpperCase(it->objectId())==TeConvertToUpperCase(object_id))
		{
			val = it->getPropertyValue(attr_index);
			return true;
		}
		++it;
	}
	return false;
}

int 
TeSTEventVector::getAttributeIndex(const string& attrName)
{
	STElemVecIterator it = events_.begin();
	return(it->getPropertyIndex(attrName));
}


void 
TeSTEventVector::iterator::operator++()
{
	if(curElemVec_!= eventVector_->events_.end())
		++curElemVec_;
}

TeSTEventSetImpl::iterator*  
TeSTEventVector::begin()     
{ 
	TeSTEventSetImpl::iterator* it = new TeSTEventVector::iterator(events_.begin(), this);
	return it;
}


TeSTEventSetImpl::iterator* 
TeSTEventVector::end()       
{
	TeSTEventSetImpl::iterator* it = new TeSTEventVector::iterator(events_.end(), this);
	return it;
}

void 
TeSTEventVector::insertSTInstance(TeSTEvent& ev)
{ 
	events_.push_back(ev);
}

bool 
TeSTEventVector::buildKDTree(const int& n)
{
	vector<pair<TeCoord2D, TePoint> >	dataSetAux; //auxiliary structure for kdTree
	STElemVecIterator it = events_.begin();
	int index=0;
	while(it!=events_.end())
	{
		TePoint p = (*it).point();
		p.geomId(index);
		dataSetAux.push_back(pair<TeCoord2D, TePoint>(p.location(), p));
		++it;
		++index;
	}

	int bucketSize; 
	if(n<0)
		bucketSize= 30;
	else
		bucketSize= 2*n;
	eventTree_ = new kdTree(box_, bucketSize);
	eventTree_->build(dataSetAux);
	kdTreeBuilt_=true;
	return true;
}

void 
TeSTEventVector::clear() 
{ 
	events_.clear(); 
	if(eventTree_)
		eventTree_->clear();
}

TeSTEvent 
TeSTEventVector::operator[](const string& id)
{
	STElemVecIterator it = events_.begin();
	while(it!=events_.end())
	{
		if(TeConvertToUpperCase(it->objectId())==TeConvertToUpperCase(id))
			return (*it);
		++it;
	}
	return TeSTEvent();
}

//------------------------------------------ TeSTEventMap
TeSTEventMap::TeSTEventMap(): TeSTEventSetImpl()
{}

TeSTEventMap::TeSTEventMap(const TeBox& box): TeSTEventSetImpl(box)
{}

bool 
TeSTEventMap::getAttributeValue (const string& object_id, const int& attr_index, double& val, const long& /*time*/)
{
	STElemMapIterator it = events_.find(object_id);
	if(it==events_.end())
		return false;

	val = (it->second).getPropertyValue(attr_index);
	return true;
}

int 
TeSTEventMap::getAttributeIndex(const string& attrName)
{
	STElemMapIterator it = events_.begin();
	return(it->second.getPropertyIndex(attrName));
}


void 
TeSTEventMap::iterator::operator++()
{
	if(curElemMap_!= eventMap_->events_.end())
		++curElemMap_;
}

TeSTEventSetImpl::iterator* 
TeSTEventMap::begin()     
{ 
	TeSTEventSetImpl::iterator* it = new TeSTEventMap::iterator(events_.begin(), this);
	return it;
}


TeSTEventSetImpl::iterator*  
TeSTEventMap::end()       
{
	TeSTEventSetImpl::iterator* it = new TeSTEventMap::iterator(events_.end(), this);
	return it;
}

void 
TeSTEventMap::insertSTInstance(TeSTEvent& ev)
{ 
	events_[ev.objectId()]=ev;
}

bool 
TeSTEventMap::buildKDTree(const int& n)
{
	vector<pair<TeCoord2D, TePoint> >	dataSetAux; //auxiliary structure for kdTree
	STElemMapIterator it = events_.begin();
	while(it!=events_.end())
	{
		dataSetAux.push_back(pair<TeCoord2D, TePoint>(it->second.point().location(), it->second.point()));
		++it;
	}

	int bucketSize; 
	if(n<0)
		bucketSize= 30;
	else
		bucketSize= 2*n;
	eventTree_ = new kdTree(box_, bucketSize);
	eventTree_->build(dataSetAux);
	kdTreeBuilt_ = true;
	return true;
}

void 
TeSTEventMap::clear() 
{ 
	events_.clear(); 
	if(eventTree_)
		eventTree_->clear();
}

TeSTEvent 
TeSTEventMap::operator[](const unsigned int& index)
{
	if(index>(events_.size()-1))
		return TeSTEvent();
	
	STElemMapIterator it = events_.begin();
	unsigned int i=0;
	while (it!=events_.end())
	{
		if(i==index)
			return (it->second);
		++it;
		++i;
	}
	return TeSTEvent();
}

TeSTEvent 
TeSTEventMap::operator[](const string& id)
{
	STElemMapIterator it = events_.find(id);
	if(it==events_.end())
		return TeSTEvent();
	return (it->second);
}

//------------------------------------------ TeSTEventSet
TeSTEventSet::TeSTEventSet(TeTheme* theme, bool objIdOrdered): 
		events_(0),
		theme_(theme),
		orderedByObjId_(objIdOrdered)
{ 
		
	if(theme)
	{
		if(objIdOrdered)
			events_ = new TeSTEventMap(theme->getThemeBox());
		events_ = new TeSTEventVector(theme->getThemeBox());
	}
}

TeSTEventSet::~TeSTEventSet()
{ 
	if(events_)
		delete(events_);
}

TeTheme* 
TeSTEventSet::theme()
{	
	return theme_; 
}

void 
TeSTEventSet::theme(TeTheme* t)
{
	theme_ = t;
}

void 
TeSTEventSet::box(const TeBox& b)
{ 
	if(events_)
		events_->setBox(b);
} 

TeBox 
TeSTEventSet::box() 
{ 
	if(events_)
		return (events_->box());
	return TeBox();
}

unsigned int 
TeSTEventSet::size()
{ 
	if(events_)
		return events_->size();
	return 0;
}

bool
TeSTEventSet::getAttributeValue (const string& object_id, const string& attr_name, string& val, TeTimeInterval /* time */)
{
	int index = getAttributeIndex(attr_name);
	if(index<0)
		return false;
	
	double vald = 0.;
	bool status =  getAttributeValue (object_id,  index, vald);
	val = Te2String(vald);
	return status;
}
	
bool 
TeSTEventSet::getAttributeValue (const string& object_id, const int& attr_index, string& val, TeTimeInterval /* time */)
{
	double vald = 0.;
	bool status =  getAttributeValue (object_id,  attr_index, vald);
	val = Te2String(vald);
	return status;
}


bool 
TeSTEventSet::getAttributeValue (const string& object_id, const int& attr_index, double& val, const long& time )
{
	if(!events_)
		return false;
	return (events_->getAttributeValue (object_id, attr_index, val,time));
}

int 
TeSTEventSet::getAttributeIndex(const string& attrName)
{
	if(!events_)
		return -1;
	return (events_->getAttributeIndex (attrName));
}

int 
TeSTEventSet::numSTInstance()
{ 
	if(events_)
		return events_->size();
	return 0;
}

int 
TeSTEventSet::numElements()
{ 
	if(events_)
		return events_->size();
	return 0;
}

bool 
TeSTEventSet::search(const TeBox& b, vector<TePoint>& result)
{
	if(!events_)
		return false;
	return events_->search(b, result);
}
	
bool 
TeSTEventSet::nearestNeighbourSearch(const TeCoord2D& coord, vector<TePoint>& result, vector<double>& distances, const unsigned int& k)
{
	if(!events_)
		return false;
	return events_->nearestNeighbourSearch(coord, result, distances, k);
}


TeSTEventSet::iterator& 
TeSTEventSet::iterator::operator++() 
{
	it_->operator++(); 
	return (*this);
}

TeSTEventSet::iterator&  
TeSTEventSet::iterator::operator=(const iterator& rhs)
{
	if ( this != &rhs )
	{
		elemSet_ = rhs.elemSet_;	
		it_ = elemSet_->events_->begin(); //return a iterator
		(*it_)=(*(rhs.it_));
	}
	return *this;
}

TeSTEvent& 
TeSTEventSet::iterator::operator*()
{	
	return (it_->operator*()); 	
}

bool 
TeSTEventSet::iterator::operator==(const iterator& rhs) const
{ 
	return (it_->operator==(rhs.it_)); 
}

bool 
TeSTEventSet::iterator::operator!=(const iterator& rhs) const
{ 
	return (it_->operator!=(rhs.it_)); 
}

TeSTEventSet::iterator 
TeSTEventSet::begin()     
{ 
	TeSTEventSetImpl::iterator*	it = events_->begin();
	return iterator(this, it);
}


TeSTEventSet::iterator 
TeSTEventSet::end()       
{
	TeSTEventSetImpl::iterator*	it = events_->end();
	return iterator(this, it);
}

bool  
TeSTEventSet::insertSTInstance(TeSTEvent& ev)
{ 
	if(!events_)
		return false;
	events_->insertSTInstance(ev);
	return true;
}

bool 
TeSTEventSet::insertSTInstance (TeSTInstance& object)
{
	TeCoord2D coord;
	object.centroid(coord);
	vector<double> values;
	vector<TeAttributeRep> reps;
	TePropertyVector::iterator it = object.getPropertyVector().begin();
	while(it!=object.getPropertyVector().end())
	{
		if((*it).attr_.rep_.type_==TeINT || (*it).attr_.rep_.type_==TeREAL)
		{
			values.push_back (atof((*it).value_.c_str()));
			reps.push_back ((*it).attr_.rep_);
		}
		++it;
	}

	TeSTEvent ev(object.objectId(), TePoint(coord), values, reps, -1);
	return (this->insertSTInstance(ev));
}

bool
TeSTEventSet::buildKDTree(const int& n)
{ 
	if(!events_)
		return false;
	return (events_->buildKDTree(n));
}

void 
TeSTEventSet::clear() 
{ 
	if(!events_)
		return;
	events_->clear();
}

TeSTEvent 
TeSTEventSet::operator[](const unsigned int& index)
{
	TeSTEvent ev; 
	if(!events_ || orderedByObjId_)
		return ev;

	return (events_->operator[](index));
}










